/*-----------------------------------------------------*/
/*     Name:               astro.h                     */
/*-----------------------------------------------------*/
/*  Aufgabe: Makros, Prototypen, Strukturen usw.       */
/*-----------------------------------------------------*/
/*   Copyright (C) 1993 - 2003 by Michael Schroeder    */
/*-----------------------------------------------------*/


/*
 * LIBASTRO is distributed under the GNU GENERAL PUBLIC LICENSE (GPL)
 * Version 2 (June 1991). See the "COPYING" file distributed with this
 * software for more info.
 */

/*
 * Meine Koordinaten: -10.47036 Ost, +52.27814 Nord
 */

#ifndef ASTRO_H
# define ASTRO_H

#include <fmath.h>	/* sonst geht nichts */

#if __cplusplus
extern "C" {
#endif


/**************/
/* Konstanten */
/**************/
#define UNIT_DIST	149597870660.0	/* Astron. Entfern.einheit [m]	*/
#define UNIT_MASS	1.9891E+30	/* Astron. Masseeinheit [kg]	*/
#define UNIT_TIME	86400.0		/* Astron. Zeiteinheit [s]	*/

#define CONST_C		299792458.0	/* Lichtgeschwindigkeit [m/s]	*/
#define CONST_GAUSS	0.01720209895	/* Gausssche Gravitationskonst. */
#define CONST_LTIME	499.004782	/* Lichtzeit fuer 1 AU [s], IAU	*/
#define CONST_SIDER	86164.09054	/* Dauer eines Sterntags [s]	*/
#define CONST_ABERR	9.9365085E-5	/* Aberrationskonst. (20.49552")*/
	
#define CONST_GRAV	(CONST_GAUSS * CONST_GAUSS)
#define CONST_LIGHT	(UNIT_TIME / CONST_LTIME)
#define CONST_LIMIN	(CONST_LTIME / 60.0)
#define CONST_PARSEC	ARCS_PER_RAD
#define CONST_DIURNAL	(2 * F_PI * RADI_EARTH * CONST_LTIME / CONST_SIDER)
#define CONST_LTCOR	(CY_PER_DAY / CONST_LIGHT)
#define CONST_PARALLAX	RADI_EARTH

#define EPOC_B1900	2415020.31352	/* JD(1900 Jan 0.81352) 	*/
#define EPOC_B1950	2433282.42346	/* JD(1950 Jan 0.92346) 	*/
#define EPOC_B2000	2451544.53340	/* JD(2000 Jan 1.03340) 	*/
#define EPOC_J1900	2415020.0	/* JD(1900 Jan 0.5)		*/
#define EPOC_J1950	2433282.5	/* JD(1950 Jan 1.0)		*/
#define EPOC_J2000	JD2000		/* JD(2000 Jan 1.5)		*/
#define EPOC_J2050	2469807.5	/* JD(2050 Jan 1.0)		*/

#define FLAT_EARTH	0.00335281	/* Erdabplattung (1:298.257)	*/
#define FLAT_JUPITER	0.06480881	/* Jupiterabplattung (1:15.43)	*/
#define FLAT_MARS	0.00518645	/* Marsabplattung (1:192.81)	*/
#define FLAT_MERCURY	0.0
#define FLAT_MOON	0.0
#define FLAT_NEPTUNE	0.0259		/* Neptunabplattung (1:38.61)	*/
#define FLAT_PLUTO	0.0
#define FLAT_SATURN	0.1076209	/* Saturnabplattung (1:9.29187)	*/
#define FLAT_SUN	0.0
#define FLAT_URANUS	0.030		/* Uranusabplattung (1:33)	*/
#define FLAT_VENUS	0.0
						/* Quelle: DE118i	*/
#define MASS_EARTH	3.003489712548569096E-6	/* Erdmasse [Sol]	*/
#define MASS_EARTH_MOON	3.040432738710834925E-6	/* Erd+Mondmasse [Sol]	*/
#define MASS_JUPITER	9.547906522055788602E-4	/* Jupitermasse [Sol]	*/
#define MASS_MARS	3.227149362153928503E-7	/* Marsmasse [Sol]	*/
#define MASS_MERCURY	1.660136795271930376E-7	/* Merkurmasse [Sol]	*/
#define MASS_MOON	3.694302616226582840E-8	/* Mondmasse [Sol]	*/
#define MASS_NEPTUNE	5.177591320617641350E-5	/* Neptunmasse [Sol]	*/
#define MASS_PLUTO	7.692307678184215319E-9	/* Plutomasse [Sol]	*/
#define MASS_SATURN	2.858776443811418597E-4	/* Saturnmasse [Sol]	*/
#define MASS_SUN	1.0			/* Sonnenmasse [Sol]	*/
#define MASS_URANUS	4.355400696730197767E-5	/* Uranusmasse [Sol]	*/
#define MASS_VENUS	2.447839597966824381E-6	/* Venusmasse [Sol]	*/

#define RADI_EARTH	(6378137/UNIT_DIST)	/* Erdradius [AU]	*/
#define RADI_JUPITER	(71492000/UNIT_DIST)	/* Jupiterradius [AU]	*/
#define RADI_MARS	(3397000/UNIT_DIST)	/* Marsradius [AU]	*/
#define RADI_MERCURY	(2439700/UNIT_DIST)	/* Merkurradius [AU]	*/
#define RADI_MOON	(1738200/UNIT_DIST)	/* Mondradius [AU]	*/
#define RADI_NEPTUNE	(24764000/UNIT_DIST)	/* Neptunradius [AU]	*/
#define RADI_PLUTO	(1151000/UNIT_DIST)	/* Plutoradius [AU]	*/
#define RADI_SATURN	(60268000/UNIT_DIST)	/* Saturnradius [AU]	*/
#define RADI_SUN	(696000000/UNIT_DIST)	/* Sonnenradius [AU]	*/
#define RADI_URANUS	(25559000/UNIT_DIST)	/* Uranusradius [AU]	*/
#define RADI_VENUS	(6051900/UNIT_DIST)	/* Venusradius [AU]	*/

#define YEAR_JULIAN	365.25		/* Laenge des julianischen Jahres*/
#define YEAR_TROPIC	365.2421896698	/* Laenge des tropischen Jahres	*/

#define MONTH_SYNODIC	29.5305888531	/* Laenge des synodischen Monats*/

#define EPS2000		0.409092804202936390	/* epsilon(J2000)	*/ 
#define COSE2000	0.917482062076895738	/* cos(EPS2000)		*/ 
#define SINE2000	0.397777155914121387	/* sin(EPS2000)		*/

#define DEGREE		(180.0 / F_PI)
#define RADIAN		(F_PI / 180.0)

#define JD2000		2451545.0		/* JD von J2000.0	*/
#define DAY_PER_CY	(100.0 * YEAR_JULIAN)	/* Tage pro Jahrhundert	*/
#define SEC_PER_CY	(DAY_PER_CY * UNIT_TIME)
#define SEC_PER_DAY	UNIT_TIME
#define CY_PER_DAY	(1.0 / DAY_PER_CY)
#define CY_PER_SEC	(1.0 / SEC_PER_CY)
#define DAY_PER_SEC	(1.0 / SEC_PER_DAY)
#define MIN_PER_DAY	1440.0
#define HOUR_PER_RAD	(12.0 / F_PI)
#define RAD_PER_HOUR	(F_PI / 12.0)
#define ARCS_PER_RAD	(3600.0 / RADIAN)
#define RAD_PER_ARCS	(RADIAN / 3600.0)
#define SEC_PER_RAD	(240.0 / RADIAN)
#define RAD_PER_SEC	(RADIAN / 240.0)

/**********/
/* System */
/**********/
#define ARG_0		0x00		/* Zur besseren Lesbarkeit	*/
#define ARG_1		0x01
#define ARG_2		0x02
#define ARG_3		0x03
#define ARG_4		0x04
#define ARG_5		0x05
#define ARG_6		0x06
#define ARG_7		0x07
#define ARG_8		0x08
#define ARG_9		0x09
#define ARG_A		0x0a
#define ARG_B		0x0b



/***************/
/* Zeichensatz */
/***************/
#define FNT_MOUSEPTR	'\x07'
#define FNT_MOONNEW	'\x01'
#define FNT_MOONFULL	'\x02'
#define FNT_MOONFQT	'\x03'
#define FNT_MOONLQT	'\x04'
#define FNT_SUN		'\x80'
#define FNT_MERCURY	'\x86'
#define FNT_VENUS	'\x89'
#define FNT_EARTH	'\x86'
#define FNT_MARS	'\x8f'
#define FNT_JUPITER	'\x91'
#define FNT_SATURN	'\x92'
#define FNT_URANUS	'\x98'
#define FNT_NEPTUNE	'\x9b'
#define FNT_PLUTO	'\x9c'
#define FNT_IDXQUOT	'\x9d'
#define FNT_HOUR	'\xa4'
#define FNT_MINUTE	'\xa5'
#define FNT_SECOND	'\xa6'
#define FNT_IDXV	'\xa7'
#define FNT_IDXH	'\xa8'
#define FNT_IDXM	'\xa9'
#define FNT_IDXO	'\xaa'
#define FNT_IDXS	'\xab'
#define FNT_IDXHELIO	'\xac'
#define FNT_IDXGEO	'\xad'
#define FNT_DIAMETER	'\xed'
#define FNT_ALPHA	'\xe0'
#define FNT_UALPHA	'\x41'
#define FNT_BETA	'\xe1'
#define FNT_UBETA	'\x42'
#define FNT_GAMMA	'\xec'
#define FNT_UGAMMA	'\xe2'
#define FNT_DELTA	'\xeb'
#define FNT_UDELTA	'\xf0'
#define FNT_EPSILON	'\xee'
#define FNT_UEPSILON	'\x45'
#define FNT_ZETA	'\xf6'
#define FNT_UZETA	'\x5a'
#define FNT_ETA		'\xf4'
#define FNT_UETA	'\x48'
#define FNT_THETA	'\xf7'
#define FNT_UTHETA	'\xe9'
#define FNT_IOTA	'\x69'
#define FNT_UIOTA	'\x49'
#define FNT_LAMBDA	'\xef'
#define FNT_ULAMBDA	'\xff'
#define FNT_MY		'\xe6'
#define FNT_UMY		'\x4d'
#define FNT_NY		'\x76'
#define FNT_UNY		'\x4e'
#define FNT_XI		'\xf5'
#define FNT_UXI		'\xf9'
#define FNT_OMIKRON	'\x6f'
#define FNT_UOMIKRON	'\x4f'
#define FNT_PI		'\xe3'
#define FNT_UPI		'\xfb'
#define FNT_RHO		'\xf1'
#define FNT_URHO	'\x50'
#define FNT_SIGMA	'\xe5'
#define FNT_USIGMA	'\xe4'
#define FNT_TAU		'\xe7'
#define FNT_UTAU	'\x54'
#define FNT_YPSILON	'\xf3'
#define FNT_UYPSILON	'\x59'
#define FNT_PHI		'\xf2'
#define FNT_UPHI	'\xe8'
#define FNT_CHI		'\x78'
#define FNT_UCHI	'\x58'
#define FNT_PSI		'\xfa'
#define FNT_UPSI	'\xfc'
#define FNT_OMEGA	'\xfe'
#define FNT_UOMEGA	'\xe8'

typedef enum		/* Griechische Buchstaben */
{
  GE_NON = 0,
  GE_ALP,
  GE_BET,
  GE_GAM,
  GE_DEL,
  GE_EPS,
  GE_ZET,
  GE_ETA,
  GE_THE,
  GE_IOT,
  GE_KAP,
  GE_LAM,
  GE_MU,
  GE_NU,
  GE_XI,
  GE_OMI,
  GE_PI,
  GE_RHO,
  GE_SIG,
  GE_TAU,
  GE_UPS,
  GE_PHI,
  GE_CHI,
  GE_PSI,
  GE_OME,
} GREEKCHAR;


/****************/
/* INI-Keywords */
/****************/
#define INI_OBS_LONG	"GEOLAENGE"		/* Geogr. Laenge d. Beobacht.*/
#define INI_OBS_LATIT	"GEOBREITE"		/* Geogr. Breite d. Beobacht.*/
#define INI_OBS_HEIGHT	"GEOHOEHE"		/* Hoehe ueNN d. Beobacht. [m] */

#define INI_TIME_START	"STARTEPOCHE"		/* Startepoche in [JD] */

#define INI_DELTAT	"TD-UT"
#define INI_FONT	"FONT"
#define INI_MOUSEPTR	"POINTER"
#define INI_MPTRSOFT	"SOFT"

#define INI_TEMPERAT	"TEMPERATUR"
#define INI_PRESSURE	"LUFTDRUCK"

#define INI_PRECISION	"GENAUIGKEIT"
#define INI_CALREFORM	"CALREFORM"

#ifdef __EMX__
# define INI_ENVNAME	"ASTRO_INI"		/* Umgebungsvariable */
# define INI_NAME	"ASTRO.INI"		/* Name der INI-Datei */
#else
# define INI_ENVNAME	"ASTRO_RC"
# define INI_NAME	".astrorc"
#endif
#define INI_REMSIGN	';'			/* Kommentarzeichen */
#define INI_SEPSIGN	'='			/* Trennungszeichen */


/**************/
/* Strukturen */
/**************/

typedef enum		/* Genauigkeitsvorgabe fuer Funktionen		*/
{
  HIGH = 0,		/* volle Genauigkeit				*/
  MEDIUM,		/* mittlere Genauigkeit				*/
  LOW,			/* niedrige Genauigkeit				*/
} PRECISION;

#ifndef CALEND_H
typedef struct		/* Datum */
{
  long   year;
  int    month;
  double day;		/* mit Tagesbruchzahl = Stunde/24 */
} DATE,	*pDATE;
#endif


/* `Rechtwinklige Koordinaten'
 *
 * Datentyp: CARTESIC:
 *
 *                   --------------------------------------------------- 
 *                  | galaktikal | ekliptikal | aequatorial | horizontal |
 *  ----------------+------------+------------+------------+------------|
 * | x-y-Ebene      | Milchstrasse| Ekliptik   | Aequator    | Horizont   |
 * | z-Achse        | Galak. Pol | Eklptikpol | Nordpol    | Zenit      |
 * | Nullpunkt (x-y)| Gal.Zentrum| Fruehl.pkt  | Fruehl.pkt  | Suedmeridian|
 *  -------------------------------------------------------------------- 
 */

/* `Kugelkoordinaten'
 *
 * Datentyp: SPHERIC:
 *                               Bezugsebene des Grosskreises
 *                   --------------------------------------------------- 
 *    Mittelpunkt   | galaktikal | ekliptikal | aequatorial | horizontal |
 *  ----------------+------------+------------+------------+------------|
 * | heliozentrisch |            |  l,  b, r  |            |            |
 * | geozentrisch   |  l, b, (r) | la, be, r  | ra, de, d  |            |
 * | topozentrisch  |            |            | ra, de, d  |  A, h, (d) |
 *  -------------------------------------------------------------------- 
 *
 * - Um die Programme besser lesbar zu machen, wird eine einheitliche
 *   Koordinaten-Struktur verwendet.
 * - Der Zugriff erfolgt immer ueber:
 *	`.r' die Entfernung vom Ursprung (Radiusvektor),
 *	`.l' der Winkel am Grosskreis (Laenge),
 *	`.b' der Winkelabstand von der Grosskreisebene (Breite);
 *   (die Reihenfolge in der Struktur wurde `mathematisch' gewaehlt).
 */

/*
 * Zustandsvektor
 */
typedef struct
{
  VECTOR r;		/* Ortskoordinate		*/
  VECTOR v;		/* Momentan-Geschwindigkeit	*/
} STATVEC, *pSTATVEC;



/*
 * In den nachfolgenden Strukturen werden alle! Groessen wie folgt gemessen:
 *
 * - Zeiten: in Jahrhunderten seit J2000.0
 * - Laengen: in Astronomischen Einheiten
 * - Winkel: in Radiant (Bogenmass)
 */
typedef struct	/* Mittl. Argumente des Erde-Mond-Sonne Systems		*/
{
  double td;		/* Dynamische Zeit (hier gelte TD = TDB = TDT)	*/
  double ms;		/* Mittlere Anomalie der Sonne			*/
  double mm;		/* Mittlere Anomalie des Mondes			*/
  double d; 		/* Mittlere Elongation des Mondes zur Sonne	*/
  double f;		/* Mittleres Breitenargument des Mondes		*/
  double k;		/* Laenge des aufsteigenden Knotens der Mondbahn	*/
  double ls;		/* Mittlere Laenge der Sonne			*/
  double lm;		/* Mittlere Laenge des Mondes (o. Lichtlaufzeit)	*/
  double w;		/* Mittlere Perihellaenge der Erde		*/
  double e;		/* Mittlere Exzentrizitaet der Erdbahn		*/
} LUNISOL, *pLUNISOL;


typedef struct		/* Argumente der Praezession			*/
{
  double x;		/* x: zeta, 90- PI				*/
  SINCOS csx;		/* SINCOS von x					*/
  double y;		/* y: ZETA, p + PI - 90			*/
  SINCOS csy;		/* SINCOS von y					*/
  double z;		/* z: theta, pi					*/
  SINCOS csz;		/* SINCOS von z					*/
  MATRIX mat;		/* Praezessionsmatrix: Datum -> J2000.0		*/
} PRECES, *pPRECES;


typedef struct	/* Nutation						*/
{
  double de;		/* Nutation in Schiefe				*/
  double dp;		/* Nutation in Laenge				*/
  SINCOS csdp;		/* SINCOS der Nutation in Laenge			*/
  MATRIX mat;		/* Nutationsmatrix: mittel -> wahr		*/
} NUTATION, *pNUTATION;


typedef struct	/* Schiefe der Ekliptik					*/
{
  double e;		/* Mittlere Schiefe der Ekliptik		*/
  SINCOS cse;		/* Co+Sinus der mittleren Schiefe der Ekliptik	*/
  double te;		/* Wahre Schiefe der Ekliptik	(te = e + de)	*/
  SINCOS cste;		/* SINCOS der wahren Schiefe der Ekliptik	*/
} OBLIQUITY, *pOBLIQUITY;


typedef struct	/* Aequatorlage (Nutation und Ekliptikschiefe)		*/
{
  OBLIQUITY obl;	/* Zusammenfassung der Lageelemente der Erde	*/
  NUTATION  nut;
} EQUATOR, *pEQUATOR;


typedef struct	/* geozentrische Sonnenkoordinaten			*/
{
  SPHERIC  pos;	 /* sphaerisch geozentrisch ekliptikal (Datum)		*/
  CARTESIC rec;	 /* rechtwinklig geozentrisch ekliptikal (Datum)	*/
  CARTESIC rec2; /* rechtwinklig geozentrisch ekliptikal (J2000)	*/
  CARTESIC req;	 /* rechtwinklig geozentrisch aequatorial (Datum)	*/
  CARTESIC req2; /* rechtwinklig geozentrisch aequatorial (J2000)	*/
} SOLARCO, *pSOLARCO;


typedef struct	/* baryzentrische Zustandsvektoren der Erde		*/
{
  STATVEC ecl;	/* rechtwinklig baryzentrisch ekliptikal (Datum)	*/
  STATVEC ecl2;	/* rechtwinklig baryzentrisch ekliptikal (J2000)	*/
  STATVEC equ;	/* rechtwinklig baryzentrisch aequatorial (Datum)	*/
  STATVEC equ2;	/* rechtwinklig baryzentrisch aequatorial (J2000)	*/
} BARYSTAT, *pBARYSTAT;

  
typedef struct	/* Zeitgroessen						*/
{
  double dt;		/* Differenz von TD und UT: DT = TD - UT	*/
  double ut;		/* Weltzeit UT (UTC ist periodisch, UT nicht!)	*/
  double gmst;		/* Mittlere Greenwich-Sternzeit			*/
  double gast;		/* Wahre Greenwich-Sternzeit 			*/
  double last;		/* Wahre Ortssternzeit				*/
  SINCOS csl;		/* SINCOS der wahren Ortssternzeit		*/
}  TIME, *pTIME;


typedef struct	/* Vom Beobachter abhaengige Groessen			*/
{
  SPHERIC  pos;		/* sphaerische geografische Beobachterkoord.	*/
  SINCOS   csb;		/* SINCOS der geographischen Breite		*/ 
  double   phi;		/* phi': geozentrische Breite des Beobachters	*/
  RECTAN   rec;		/* rechtw. geozentrische Beobachterkoordinaten	*/
  CARTESIC ecl;		/* ekliptikale geozentrische Beobachterkoord.	*/
  CARTESIC equ;		/* aequatoriale geozentrische Beobachterkoord.	*/
} OBSERVER, *pOBSERVER;


typedef struct 		/* Wichtige Groessen des Erde-Mond-Sonne Systems	*/
{
  LUNISOL	ems;	/* Argumente des Erde-Mond-Sonne Systems	*/
  PRECES	pre;	/* Argumente der Praezession (aequatorial)	*/
/* EQUATOR	equ;	Reihenfolge OBLIQUITY-NUTATION beibehalten!	*/
  OBLIQUITY	obl;	/* Ekliptikschiefe				*/
  NUTATION	nut;	/* Nutation					*/
  SOLARCO	sun;	/* geozentrische Sonnenkoordinaten		*/
  BARYSTAT	bar;	/* barizentrische Zustandsvektoren der Erde	*/ 
  TIME		tim;	/* Zeitgroessen					*/
  OBSERVER	obs;	/* Beobachterdaten				*/
} FUNDAMENT, *pFUNDAMENT;


/*
 * Weitere Strukturen
 */
typedef struct		/* Besselsche Tageszahlen */
{
  double a;
  double b;
  double c;
  double d;
  double e;
  double mn;		/* (m/n) */
  double te;		/* tan '*/
} BESDAYNUM, *pBESDAYNUM;


typedef struct		/* Besselsche Sternkonstanten */
{
  double a;
  double b;
  double c;
  double d;
  double aa;
  double bb;
  double cc;
  double dd;
} BESSTARCONST, *pBESSTARCONST;


/* 
 * Verwaltung der Objekte, Transformationen, Reduktionen usw...
 */
typedef enum		/* Art der Reduktionen ...		*/
{
  ADDON	= 0,		/* hinzufuegen */
  FREE	= 1,		/* befreien */
} CORRECTION;


/*
 * Abplattung(Galaxie, Nebel): 0 = kreisfoermig ... 9 = extrem gestreckt
 */
#define FLATNESS(a,b)	((10 * (a - b)) / a)	/* 0 - 9 */

typedef enum		/* Himmelsobjekt-Identifizierer		*/
{			/* auch fuer Sternkataloge (NGC, IC, M)	*/
  EARTH		= 0,
  BARYCENTER,		/* planetares Massenzentrum		*/
  EARTHMOON,		/* Schwerpunkt im Erde-Mond-System	*/
  SUN,
  MERCURY,		/* Reihenfolge nicht aendern!    	*/ 
  VENUS,		/* Abfragen werden dadurch einfacher	*/
  MARS,			/* Bsp.: `case MERCURY ... PLUTO'	*/
  JUPITER,
  SATURN,
  URANUS,
  NEPTUNE,
  PLUTO,
  MOON,
  ASTEROID,
  COMET,

  STAR		= 0x40,
  STAR_SINGLE,
  STAR_DOUBLE,		/* Doppelstern				*/
  STAR_TRIPLE,		/* Dreifachstern			*/

  PLANET_NEBULA	= 0x50,	/* planetarischer Nebel			*/
  PLANET_NEBUL_ANN,	/* ringfoermiger planetarischer Nebel	*/
  PLANET_NEBUL_CIR,	/* kreisfoermiger planetarischer Nebel	*/
  PLANET_NEBUL_IRR,	/* unregelmaessiger planetarischer Nebel*/

  NEBULA	= 0x60,	/* Gasnebel				*/
  NEBULA_BRI,		/* heller Gasnebel			*/
  NEBULA_EMM,		/* Emmisionsnebel			*/
  NEBULA_RFL,		/* Reflektionsnebel			*/
  NEBULA_FIB,		/* faseriger, filamentartiger Nebel	*/
  NEBULA_EXT,		/* Nebelregion in anderer Galaxie	*/

  OPEN_CLUSTER	= 0x80,	/* offener Sternhaufen			*/
  OPEN_CLUSTER_RI,	/* reicher offener Sternhaufen		*/
  OPEN_CLUSTER_ME,	/* mittlerer offener Sternhaufen	*/
  OPEN_CLUSTER_PO,	/* armer offener Sternhaufen		*/
  OPEN_CLUSTER_VP,	/* Gruppe von sehr wenigen Sternen	*/
  OPEN_CLUSTER_NB,	/* offener Sternhaufen mit Gasnebel	*/

  GLOB_CLUSTER	= 0x90,	/* kugelfoermiger Sternhaufen		*/
  GLOB_CLUSTER_I,	/* heller kleiner Kern			*/
  GLOB_CLUSTER_II,
  GLOB_CLUSTER_III,
  GLOB_CLUSTER_IV,
  GLOB_CLUSTER_V,
  GLOB_CLUSTER_VI,
  GLOB_CLUSTER_VII,
  GLOB_CLUSTER_VIII,
  GLOB_CLUSTER_IX,
  GLOB_CLUSTER_X,
  GLOB_CLUSTER_XI,
  GLOB_CLUSTER_XII,	/* gleichmaessiges Zentralgebiet	*/

  GALAXY	= 0xA0,	/* Galaxie				*/
  GALAXY_ELLIP,
  GALAXY_LENTI,
  GALAXY_SPIRAL,
  GALAXY_IRREG,


  BODY_FOTO_DEF	= 0xf0,	/* Fehler auf Fotoplatte		*/
  BODY_UNCERTAIN= 0xfd,	/* fraglich, unsicher			*/
  BODY_UNKNOWN	= 0xfe, /* unbekannt				*/
  BODY_NONEXIST	= 0xff,	/* existiert nicht			*/
} BODY;

#define ISPLANET(b)	(b >= MERCURY && b <= COMET)
#define ISINTERN(b)	(b <= MOON)

/*
 * Verwaltung der Koordinatensysteme
 *
 * Ursprung:	helio = 0x0000,  geo = 0x0100, topo = 0x0200, bary = 0x0300
 * Ebene:	 ecli = 0x0000, equa = 0x0010, hori = 0x0020, gala = 0x0020
 * Typus:	 mean = 0x0000, true = 0x0001, appa = 0x0002
 *
 * Beschreiber = Ursprung | Ebene | Typus
 */
#define C_MEAN		0x0000
#define C_TRUE		0x0001
#define C_APPA		0x0002
#define C_ECLI		0x0000
#define C_EQUA		0x0010
#define C_HORI		0x0020
#define C_GALA		0x0030
#define C_HELIO		0x0000
#define C_GEO		0x0100
#define C_TOPO		0x0200
#define C_BARY		0x0300

#define C_CENTER	0x0f00		/* Maske fuer Ursprung	*/
#define C_PLANE		0x00f0		/* Maske fuer Ebene	*/
#define C_TYPE		0x000f		/* Maske fuer Typus	*/

#define ISGEO(c)	(((c) & C_CENTER) == C_GEO)
#define ISHELIO(c)	(((c) & C_CENTER) == C_HELIO)
#define ISTOPO(c)	(((c) & C_CENTER) == C_TOPO)
#define ISBARY(c)	(((c) & C_CENTER) == C_BARY)
#define ISECLI(c)	(((c) & C_PLANE) == C_ECLI)
#define ISEQUA(c)	(((c) & C_PLANE) == C_EQUA)
#define ISHORI(c)	(((c) & C_PLANE) == C_HORI)
#define ISGALA(c)	(((c) & C_PLANE) == C_GALA)
#define ISMEAN(c)	(((c) & C_TYPE) == C_MEAN)
#define ISTRUE(c)	(((c) & C_TYPE) == C_TRUE)
#define ISAPPA(c)	(((c) & C_TYPE) == C_APPA)

typedef enum		/* Beschreibung der Koordinatensysteme */
{
  HELIO_ECLI_MEAN	= (C_HELIO | C_ECLI | C_MEAN),
  HELIO_EQUA_MEAN	= (C_HELIO | C_EQUA | C_MEAN),
  GEO_ECLI_MEAN		= (  C_GEO | C_ECLI | C_MEAN),
  GEO_ECLI_TRUE		= (  C_GEO | C_ECLI | C_TRUE),
  GEO_ECLI_APPA		= (  C_GEO | C_ECLI | C_APPA),
  GEO_EQUA_MEAN		= (  C_GEO | C_EQUA | C_MEAN),
  GEO_EQUA_TRUE		= (  C_GEO | C_EQUA | C_TRUE),
  GEO_EQUA_APPA		= (  C_GEO | C_EQUA | C_APPA),
  TOPO_ECLI_MEAN	= ( C_TOPO | C_ECLI | C_MEAN),
  TOPO_EQUA_MEAN	= ( C_TOPO | C_EQUA | C_MEAN),
  TOPO_EQUA_TRUE	= ( C_TOPO | C_EQUA | C_TRUE),
  TOPO_EQUA_APPA	= ( C_TOPO | C_EQUA | C_APPA),
  TOPO_HORI_APPA	= ( C_TOPO | C_HORI | C_APPA),
  BARY_ECLI_MEAN	= ( C_BARY | C_ECLI | C_MEAN),
  BARY_EQUA_MEAN	= ( C_BARY | C_EQUA | C_MEAN),
} COORD;

typedef enum		/* Verwaltung der Epochen */
{
  E_DATE  = 0,
  E_J2000,
  E_B1950,
  E_OTHER,
} EPOCH;		/* TODO: diesen Datentyp ueberfluessig machen	*/


typedef struct		/* Struktur zu Steuerung der Transformationen	*/
{
  BODY   body;		/* Identifizierer des Himmelskoerpers		*/
  COORD  from;		/* Altes Koordinatensystem			*/
  COORD	 to;		/* Neues Koordinatensystem			*/
  EPOCH  oldepoch;	/* Alte Epoche der Koordinaten			*/ 
  double old;		/* TD der alten Epoche				*/
  EPOCH  newepoch;	/* Neue Epoche der Koordinaten			*/
  double neu;		/* TD der neuen Epoche				*/
} TRANS, *pTRANS;

typedef struct		/* Struktur eines stellaren Objekts		*/
{
  char    *name;	/* Zeiger auf Namen, Katalogbezeichnung ....	*/
  SPHERIC pos;		/* sphaerische Koordinaten			*/
  SPHERIC pro;		/* Bewegung pro Jahrhundert (mit Radialgeschw.)	*/
  COORD   from;		/* Beschreibung des Koordinatensystems		*/
  double  epoch;	/* Epoche der Koordinaten			*/
} STELLAR, *pSTELLAR;

typedef struct		/* Form des Himmelskoerpers			*/
{
  double radi;		/* Radius [AU]					*/
  double flat;		/* Abplattung					*/
} SHAPE, *pSHAPE;

typedef enum		/* Drehsinn (Rotationsrichtung der Nordachse)	*/
{
  DIRECT = -1,		/* direkt = mathem. negativ = im Uhrzeigersinn	*/
  RETRO  = +1,		/* retrograd = mathematisch positiv		*/
} SENSE;

typedef struct		/* Rotationsgroessen				*/
{
  double l;		/* Laenge des Nordpols der Rotationsachse	*/
  double b;		/* Breite des Nordpols der Rotationsachse	*/
  double w;		/* Laenge des Nullmeridians (System I)		*/
  double dw2;		/* Korrektur zum Nullmeridian (System II)	*/
  double dw3;		/* Korrektur zum Nullmeridian (System III)	*/
  double sflat;		/* (1-f)^2, Abplattung f		 	*/
  SENSE sense;		/* Drehsinn 					*/
} ROTATION, *pROTATION;


typedef struct		/* Planetographische Groessen			*/
{
  double pa;		/* Positionswinkel der Rotationsachse		*/
  double l;		/* Planetozentrische Laenge (System I)		*/
  double b;		/* Planetozentrische Breite			*/
  double lam;		/* Planetographische Laenge (System I)		*/
  double lam2;		/* Planetographische Laenge (System II)		*/
  double lam3;		/* Planetographische Laenge (System III)	*/
  double bet;		/* Planetographische Breite			*/
} PLANGRAF, *pPLANGRAF;

typedef struct		/* klassische Bahnelemente eines Koerpers	*/
{			/* (die Masse sei hier als 0 angenommen)	*/
  double t0;		/* t0, Periheldurchgangszeit [TD]		*/
  double q;		/* q, Perihelabstand [AU]	 		*/
  double e;		/* e, numerische Exzentrizitaet			*/
  double i;		/* i, Bahnneigung				*/
  double k;		/* k, Laenge des aufsteigenden Knotens		*/
  double w;		/* w, Argument des Perihels			*/
} ELEMENT, *pELEMENT;
typedef const ELEMENT	cELEMENT, *pcELEMENT;

typedef struct		/* erweiterte Bahnelemente eines Koerpers	*/
{			
  double epoch;		/* Epoche der Bahnelemente [TD]	 		*/
/* ab hier identisch mit ELEMENT */
  double t0;		/* t0, Periheldurchgangszeit [TD]		*/
  double q;		/* q, Perihelabstand [AU]	 		*/
  double e;		/* e, numerische Exzentrizitaet			*/
  double i;		/* i, Bahnneigung				*/
  double k;		/* k, Laenge des aufsteigenden Knotens		*/
  double w;		/* w, Argument des Perihels			*/
/* ab hier folgen Erweiterungen und Redundanzen	*/
  double mass;		/* m, Masse des Koerpers [Sol]			*/
  double M;		/* M, mittlere Anomalie				*/
  double n;		/* n, mittlere taegl. Bewegung			*/
  double a;		/* a, grosse Halbachse [AU]			*/
  double equin;		/* Aequinoktium der Bahnelemente [TD]		*/
} XELEMENT, *pXELEMENT;
typedef const XELEMENT	cXELEMENT, *pcXELEMENT;

typedef struct		/* Bahnelemente von Planetoiden, Kometen...	*/
{
  char   name[24];	/* Name des Koerpers				*/
  long   id;		/* eindeutige Identifikationsnummer		*/
  void  *ptr;		/* Hilfszeiger o.ae.				*/
  float  h;		/* Helligkeitsparameter	k, H			*/
  float  gn;		/* Helligkeitsparameter	g, G			*/
  double epoch;		/* Epoche der Bahnelemente [TD]	 		*/
  ELEMENT elm;		/* klassische Bahnelemente			*/
} PLANETAR, *pPLANETAR;
typedef const PLANETAR	cPLANETAR, *pcPLANETAR;

typedef struct		/* Physische Ephemeriden			*/
{
	/* Eingabe-Parameter die zur Berechnung notwendig sind */
  BODY		body;	/* Identifizierer des Himmelskoerpers		*/
  CARTESIC	hel;	/* heliozen. ekliptikale Position (alle Koerper)*/ 
  double	radi;	/* Radius des Koerpers (Kometen und Planetoiden)*/
  float		h;	/* Absolute Helligkeit (Kometen und Planetoiden)*/
  float		gn;	/* Aktivitaetsparameter (Kometen) oder		*/
			/* Neigungsparameter (Planetoiden)		*/
 /* ab hier folgen die zu berechnenden Groessen bzw. Zwischenwerte */
  CARTESIC	ecl;	/* rechtw. geozentrische ekliptikale Position	*/ 
  CARTESIC	equ;	/* rechtw. geozentrische aequatoriale Position	*/ 
  double	dhel;	/* heliozentrische Entfernung	 		*/
  double	dgeo;	/* wahre geozentrische Entfernung 		*/
  double	requ;	/* scheinbarer Aequatorradius			*/
  double	rpol;	/* scheinbarer Polradius			*/
  double	para;	/* Aequatorial-Horizontalparallaxe		*/
  double	psan;	/* Positionswinkel der Sonne			*/
  double	elon;	/* Elongation zur Sonne				*/
  long		esym;	/* Elongationssymbole 'EWO...'			*/
  double	i;	/* Phasenwinkel					*/
  double	k;	/* Phase					*/
  double	q;	/* Beleuchtungsdefekt				*/
  double	mag;	/* Scheinbare Helligkeit in [mag]		*/
} PHYSICAL, *pPHYSICAL;


//typedef	long	REVOLUTION;		/* Rundenzaehler */

/*
 * Zeitpunkte der Jahreszeiten, Tierkreiszeichen
 */

typedef enum		/* Tierkreis: Winkeleinteilung der Ekliptik	*/
{
  Z_ARIES = 0,		/* Fruehlingspunkt				*/
  Z_TAURUS,
  Z_GEMINI,
  Z_CANCER,		/* Sommerpunkt					*/
  Z_LEO,
  Z_VIRGO,
  Z_LIBRA,		/* Herbstpunkt					*/
  Z_SCORPIO,
  Z_SAGITTARIUS,
  Z_CAPRICORN,		/* Winterpunkt					*/
  Z_AQURIUS,
  Z_PISCES,
  Z_SPRING = Z_ARIES,
  Z_SUMMER = Z_CANCER,
  Z_AUTUMN = Z_LIBRA,
  Z_WINTER = Z_CAPRICORN,
} ZODIAC;

typedef struct
{
  double spring;	/* Zeitpunkt des Maerz-Aequinoktiums		*/
  double summer;	/* Zeitpunkt des Juni-Solstitiums		*/
  double autumn;	/* Zeitpunkt des September-Aequinoktiums	*/
  double winter;	/* Zeitpunkt des Dezember-Solstitiums		*/
} SEASON, *pSEASON;

typedef struct		/* Aufgang, Durchgang und Untergang		*/
{
  double h0;		/* Standardhoehe des Objekts (Bsp: Sonne = -50')*/
  double rise;		/* Zeitpunkt des Aufgangs			*/
  double trans;		/* Zeitpunkt des Durchgangs			*/
  double set;		/* Zeitpunkt des Untergangs			*/
  double alti;		/* Kulminationshoehe				*/
} RISET, *pRISET;


typedef enum			/* Beschreiber einer Finsternis		*/
{
  PENUMBRAL	= 1,		/* Halbschatten-Mondfinsternis		*/
  PARTIAL	= 2,		/* Partielle Finsternis			*/
  TOTAL		= 3,		/* Totale Finsternis			*/
  ANNULAR	= 4,		/* Ringfoermige Sonnenfinsternis	*/
  ANNUTOTAL	= 5,		/* Ringfoermig-totale Sonnenfinsternis	*/
  NONCENTRAL	= 0x0010,	/* Nichtzentrale Sonnenfinsternis	*/
} ECLTYPE;

/*
 * Merkmale einer Finsternis
 */
typedef struct
{
  enum {SOLAR,LUNAR} e;	/* Gesuchte Finsternis (Sonnen oder Mondfinst.)	*/
  ECLTYPE type;		/* Beschreiber der Finsternis			*/
  double  tmax;		/* Zeitpunkt der maximalen Finsternis		*/
  double  g;		/*  der Finsternis				*/
  double  u;		/* u der Finsternis				*/
  double  penmax;	/* Maximale Groesse im Halbschatten (nur Mondf.)*/
  double  max;		/* Maximale Groesse im Kernschatten		*/
  double  penumbral;	/* Halbdauer der Halbschattenphase (nur Mondf.)	*/
  double  partial;	/* Halbdauer der partiellen Phase		*/
  double  total;	/* Halbdauer der totalen Phase			*/
} ECLIPSE, *pECLIPSE;


/*
 * Zeitpunkte der (Mond)phasen
 */
typedef struct
{
  double neu;		/* Zeitpunkt des Neumonds			*/
  double first;		/* Zeitpunkt des Ersten Viertels		*/
  double full;		/* Zeitpunkt des Vollmonds			*/
  double last;		/* Zeitpunkt des Letzten Viertels		*/
} PHASE, *pPHASE;

 
/*
 * Zeitpunkte der Knotendurchgaenge
 */
typedef struct
{
  double asc;		/* Aufsteigender Knoten				*/
  double des;		/* Absteigender Knoten				*/
} NODE, *pNODE;

 
/*
 * Zeitpunkt und Entfernung der Apsiden
 */
typedef struct
{
  double peri;		/* Periapsen-Entfernung				*/
  double ptime;		/* Periapsen-Zeitpunkt				*/
  double apo;		/* Apoapsen-Entfernung				*/
  double atime;		/* Apoapsen-Zeitpunkt				*/
} APSIDE, *pAPSIDE;

 
/*
 * Zeitpunkt und Wert der maximalen Deklination
 */
typedef struct
{
  double ntime;		/* Zeitpunkt der max. noerdlichen Deklination	*/
  double ndecl;		/* Wert der maximalen noerdlichen Deklination	*/
  double stime;		/* Zeitpunkt der max. suedlichen Deklination	*/
  double sdecl;		/* Wert der maximalen suedlichen Deklination	*/
} DECLMAX, *pDECLMAX;


/*
 * Zeitpunkt und Wert der Aspekte
 */
typedef struct
{
  double oppo;		/* Zeitpunkt der Opposition			*/
  double conj;		/* Zeitpunkt der Konjunktion			*/
  double inf;		/* Zeitpunkt der unteren Konjunktion		*/
  double sup;		/* Zeitpunkt der oberen Konjunktion		*/
  double east;		/* Zeitpunkt der groessten oestl. Elongation	*/
  double eloe;		/* Wert groessten oestl. Elongation		*/
  double west;		/* Zeitpunkt der groessten westl. Elongation	*/
  double elow;		/* Wert der groessten westl. Elongation		*/
} ASPECT, *pASPECT;

typedef enum		/* dieselbe Reihenfolge wie in Struktur		*/
{
  A_OPPOSITION,
  A_CONJUNCTION,
  A_CONJUNCTION_INFERIOR,
  A_CONJUNCTION_SUPERIOR,
  A_ELONGATION_EAST,
  A_ELONGATION_EAST_ANGLE,
  A_ELONGATION_WEST,
  A_ELONGATION_WEST_ANGLE,
} ASPECTS;

#define GETASPECT(a,asp)	(((double*)asp)[a])


#ifndef EXTERN
# if defined(__WIN32__) && !defined(__GNUC__)
#  define EXTERN 	extern __import
# else
#  define EXTERN	extern 
# endif
#endif

/*
 * RSX: In WIN-DLLs werden Daten als Zeiger auf den Inhalt uebergeben :-( 
 *
 * Durch die folgende Definition koennen die Daten importiert werden:
 *
 *  #define IMPORT_DLL_DATA
 *
 * Das Importieren von Konstanten ist nicht sinnvoll, kann aber
 * durch die folgende zusaetzliche! Definition erzwungen werden:
 *
 *  #define IMPORT_DLL_CONSTANTS
 */
#if !defined(__astro__) && defined(IMPORT_DLL_DATA)
# define asteroids	(*(*_asteroids))
# define this_orbit	(*this_orbit)
# define fk5		(*fk5)
# define fun		(*fun)
# define precision	(*precision)
# define mc		(*mc)
# define ms		(*ms)
# define deltat		(*deltat)
# define tzone		(*tzone)
# define tname		(*tname)
# define calref		(*calref)
# define jdcalref	(*jdcalref)
# define pressure	(*pressure)
# define temperature	(*temperature)

# define constel	(*constel)
# define dayname	(*dayname)
# define dayname_de	(*dayname_de)
# define dayname_en	(*dayname_en)
# define eclname	(*ecl_name)
# define eclname_de	(*eclname_de)
# define eclname_en	(*eclname_en)
# define mass		(*mass)
# define monname	(*monname)
# define monname_de	(*monname_de)
# define monname_en	(*monname_en)
# define planame	(*planame)
# define shape		(*shape)

# define e2000		(*e2000)

# if defined(IMPORT_DLL_CONSTANTS)	/* Windows: Konstante importieren :-o */
#  define cenday	(*cenday)
#  define censec	(*censec)
#  define daycen	(*daycen)
#  define daysec	(*daysec)
#  define horrad	(*horrad)
#  define jd2000	(*jd2000)
#  define radhor	(*radhor)
#  define radsec	(*radsec)
#  define seccen	(*seccen)
#  define secday	(*secday)
#  define secrad	(*secrad)
# else
#  define cenday	CY_PER_DAY
#  define censec	CY_PER_SEC
#  define daycen	DAY_PER_CY
#  define daysec	DAY_PER_SEC
#  define horrad	HOUR_PER_RAD
#  define jd2000	JD2000
#  define radhor	RAD_PER_HOUR
#  define radsec	RAD_PER_ARCS
#  define seccen	SEC_PER_CY
#  define secday	SEC_PER_DAY
#  define secrad	ARCS_PER_RAD
# endif
#endif

EXTERN const PLANETAR asteroids[];	/* Bahnelemente der Asteroiden	*/
EXTERN const PLANETAR *this_orbit;	/* Zeiger auf Bahnelemente	*/
EXTERN FUNDAMENT fk5;			/* Basisgroessen		*/
EXTERN FUNDAMENT *fun;			/* Zeiger auf Basisgroessen	*/
EXTERN PRECISION precision;		/* Genauigkeitsvorgabe		*/
EXTERN double mc[12][64], ms[12][64];	/* trigonometr. Rechenspeicher	*/
EXTERN double deltat;			/* TD - UT [TD]			*/
EXTERN double tzone;			/* Differenz TD - Ortszeit [TD]	*/
EXTERN const char *tname;		/* Zeiger auf Zeitzonenkuerzel	*/
EXTERN long calref;			/* Datum der Kalenderreform	*/
EXTERN long jdcalref;			/* JD der Kalenderreform	*/
EXTERN double pressure;			/* Luftdruck [hPa]		*/
EXTERN double temperature;		/* Temperatur [C]		*/

EXTERN const char constel[][4];		/* Sternbild-Kuerzel		*/
EXTERN const char *const *dayname;	/* Zeiger auf Tagesnamen	*/
EXTERN const char *const dayname_de[];	/* deutsche Tagesnamen		*/
EXTERN const char *const dayname_en[];	/* englische Tagesnamen		*/
EXTERN const char *const *eclname;	/* Zeiger auf Finsternisnamen	*/
EXTERN const char *const eclname_de[];	/* deutsche Finsternisnamen	*/
EXTERN const char *const eclname_en[];	/* englische Finsternisnamen	*/
EXTERN const double mass[];		/* Massen			*/
EXTERN const char *const *monname;	/* Zeiger auf Monatsnamen	*/
EXTERN const char *const monname_de[];	/* deutsche Monatsnamen		*/
EXTERN const char *const monname_en[];	/* englische Monatsnamen	*/
EXTERN const char *const planame[];	/* Namen der klassischen Koerper*/
EXTERN const SHAPE shape[];		/* Aequatorradien und Abplattung*/

EXTERN const SINCOS e2000;

#if defined(__astro__) || !defined(IMPORT_DLL_DATA) || defined(IMPORT_DLL_CONSTANTS)
EXTERN const double cenday;
EXTERN const double censec;
EXTERN const double daycen;
EXTERN const double daysec;
EXTERN const double horrad;
EXTERN const double jd2000;
EXTERN const double radhor;
EXTERN const double radsec;
EXTERN const double seccen;
EXTERN const double secday;
EXTERN const double secrad;
#endif

/****************************/
/*  Funktions-Prototypen    */
/****************************/

/* Koordinatenrechnung */
pSPHERIC app2tru(pSPHERIC app, pSPHERIC tru);
pSPHERIC app2tru2(pSPHERIC app, pSPHERIC tru);
pSPHERIC app2tru3(pSPHERIC app, pSPHERIC tru);
pCARTESIC annual_ecl(pCARTESIC old, pCARTESIC neu, int cor);
pCARTESIC annual_equ(pCARTESIC old, pCARTESIC neu, int cor);
pSPHERIC b2j(pSPHERIC b1950, pSPHERIC j2000);
void bdaynum(pBESDAYNUM bes);
pCARTESIC deflect_ecl(pCARTESIC old, pCARTESIC neu, int cor);
pCARTESIC deflect_equ(pCARTESIC old, pCARTESIC neu, int cor);
double delta(pSPHERIC pos);
double delta2(pSPHERIC pos);
pSPHERIC dia2top(pSPHERIC dia, pSPHERIC top);
double dip(double h);
pCARTESIC diurnal_ecl(pCARTESIC old, pCARTESIC neu, int cor);
pCARTESIC diurnal_equ(pCARTESIC old, pCARTESIC neu, int cor);
pSPHERIC ecl2equ(pSPHERIC ecl, pSPHERIC equ);
pSPHERIC ecl2equ2(pSPHERIC ecl, pSPHERIC equ);
pSPHERIC equ2ecl(pSPHERIC equ, pSPHERIC ecl);
pSPHERIC equ2ecl2(pSPHERIC equ, pSPHERIC ecl);
pSPHERIC equ2gal(pSPHERIC equ, pSPHERIC gal);
pSPHERIC equ2hor(pSPHERIC equ, pSPHERIC hor);
pCARTESIC equ_hor(pCARTESIC equ, pCARTESIC hor);
pCARTESIC fk4_fk5(pCARTESIC fk4, pCARTESIC fk5);
pCARTESIC fk5_fk4(pCARTESIC fk5, pCARTESIC fk4);
pSPHERIC gal2equ(pSPHERIC gal, pSPHERIC equ);
pMATRIX gaussvec(pMATRIX mat, pcELEMENT elm);
pSPHERIC geo2hel(pSPHERIC geo, pSPHERIC hel);
pSPHERIC geo2top(pSPHERIC equ, pSPHERIC top);
void geocoord(double phi, double H , pRECTAN obs);
double geodist(pRECTAN obs1, pRECTAN obs2);
const PLANETAR *getasteroid(const char *name);
pSPHERIC hel2geo(pSPHERIC hel, pSPHERIC geo);
pSPHERIC hor2equ(pSPHERIC hor, pSPHERIC equ);
pCARTESIC hor_equ(pCARTESIC hor, pCARTESIC equ);
pSPHERIC j2b(pSPHERIC j2000, pSPHERIC b1950);
pLUNISOL lunisol(double t, pLUNISOL arg);
void newdate(double t, pFUNDAMENT fk5);
void newobs(pOBSERVER);
void nutation(pLUNISOL lun, pNUTATION nut);
void nutation2(pLUNISOL lun, pNUTATION nut);
void nutmat(pEQUATOR equ, pMATRIX mat);
double obliquity(double t);
pSPHERIC preces(pPRECES ang, pSPHERIC old, pSPHERIC neu);
pMATRIX precmat(pPRECES ang, pMATRIX mat);
pPRECES prec_ecl(double T, double t, pPRECES ang);
pPRECES prec_ecl_j2000(double t, pPRECES ang);
pPRECES prec_equ(double T, double t, pPRECES ang);
pPRECES prec_equ_j2000(double t, pPRECES ang);
pSPHERIC promo(double t, pSTELLAR star, pSPHERIC pos);
double refrac(double h);
double refrac2(double h);
pSPHERIC top2dia(pSPHERIC top, pSPHERIC dia);
pSPHERIC top2geo(pSPHERIC top, pSPHERIC geo);
pCARTESIC transform(pTRANS trans, pCARTESIC old, pCARTESIC neu);
pSPHERIC tru2app(pSPHERIC tru, pSPHERIC app);
pSPHERIC tru2app2(pSPHERIC tru, pSPHERIC app);
pSPHERIC tru2app3(pSPHERIC tru, pSPHERIC app);
pCARTESIC veloc(double T, pCARTESIC vel);
double astrometric(pTRANS trans, double T, pCARTESIC pos);

/*
 * Um die Transformationen uebersichtlicher zu gestalten, werden Makros mit
 * charakteristischen Namen eingesetzt.
 * (Die ueberpruefung der Datentypen bleibt hierbei voll erhalten)
 */
#define bar_geo_ecl(b,g)	_subvec(b, &fun->bar.ecl.r, g)
#define bar_geo_ecl_j2000(b,g)	_subvec(b, &fun->bar.ecl2.r, g)
#define bar_geo_equ(b,g)	_subvec(b, &fun->bar.equ.r, g)
#define bar_geo_equ_j2000(b,g)	_subvec(b, &fun->bar.equ2.r, g)
#define geo_bar_ecl(g,b)	_addvec(g, &fun->bar.ecl.r, b)
#define geo_bar_ecl_j2000(g,b)	_addvec(g, &fun->bar.ecl2.r, b)
#define geo_bar_equ(g,b)	_addvec(g, &fun->bar.equ.r, b)
#define geo_bar_equ_j2000(g,b)	_addvec(g, &fun->bar.equ2.r, b)
#define geo_hel_ecl(g,h)	_subvec(g, &fun->sun.rec, h)
#define geo_hel_ecl_j2000(g,h)	_subvec(g, &fun->sun.rec2, h)
#define geo_hel_equ(g,h)	_subvec(g, &fun->sun.req, h)
#define geo_hel_equ_j2000(g,h)	_subvec(g, &fun->sun.req2, h)
#define geo_top_ecl(g,t)	_subvec(g, &fun->obs.ecl, t)
#define geo_top_equ(g,t)	_subvec(g, &fun->obs.equ, t)
#define hel_geo_ecl(h,g)	_addvec(h, &fun->sun.rec, g) 
#define hel_geo_ecl_j2000(h,g)	_addvec(h, &fun->sun.rec2, g) 
#define hel_geo_equ(h,g)	_addvec(h, &fun->sun.req, g) 
#define hel_geo_equ_j2000(h,g)	_addvec(h, &fun->sun.req2, g) 
#define top_geo_ecl(g,t)	_addvec(t, &fun->obs.ecl, g)
#define top_geo_equ(g,t)	_addvec(t, &fun->obs.equ, g)

#define ecl_equ(c,q)		_irot_x(c, &fun->obl.cse, q)
#define ecl_equ_tru(c,q)	_irot_x(c, &fun->obl.cste, q)
#define ecl_equ_j2000(c,q)	_irot_x(c, &e2000, q)
#define equ_ecl(q,c)		_rot_x(q, &fun->obl.cse, c)
#define equ_ecl_tru(q,c)	_rot_x(q, &fun->obl.cste, c)
#define equ_ecl_j2000(q,c)	_rot_x(q, &e2000, c)
#define mea_tru_ecl(m,t)	_irot_z(m, &fun->nut.csdp, t)
#define mea_tru_equ(m,t)	_rotate3(m, &fun->nut.mat, t)
#define tru_mea_ecl(t,m)	_rot_z(t, &fun->nut.csdp, m)
#define tru_mea_equ(t,m)	_irotate3(t, &fun->nut.mat, m)

#define _ecl_equ(c,a,q)		_irot_x(c, a, q)
#define _equ_ecl(q,a,c)		_rot_x(q, a, c)


/****************/
/* Zeitrechnung */
/****************/
/*
 * Ich habe versucht die Formeln so allgemein wie moeglich zu halten,
 * und alle Ausnahmen zu erfassen, so z.B. die Kalenderreform von 1582.
 * Im Jahre 1582 wird Donnerstag, 4. Oktober von Freitag, 15. Oktober
 * gefolgt. Gleichzeitig aendern sich die Regeln der Schaltjahresberech-
 * nung und die Methode zur Festlegung des Ostersonntags.
 * Die Reform wurde aber nicht sofort von allen Staaten angenommen,
 * so dass das Reformdatum variabel sein muss.
 *
 * Allgemein gelten alle Formel fuer:
 *
 *	-214748 Mar 1.5  bis  +214749 Feb 28.5
 *       -76715589.0  <=  JD  <=  +80156580.0
 *
 * was mehr als ausreichend sein sollte (signed long; wer mehr braucht,
 * muss auf `long long' oder `(long) double' umsteigen)
 *
 * Die Julianische Tageszaehlung JD beginnt -4712 Jan 1.5 -> (JD = 0)
 * Die dynamische Zeit TD beginnt 2000 Jan 1.5 (J2000.0), dabei gilt:
 * 	TD = (JD-2451545)/36525
 *
 * Zur Beschreibung des zukuenfigen Unterschieds zwischen TD und UT
 * wird folgende Annahme gemacht:
 *	Im Jahr 2000.0 betraegt DT = TD - UT = +64s und
 *	nimmt pro Jahrhundert um +76s zu. (Gezeitenreibung etc.)
 */

#define JD_MIN	-76715589
#define JD_MAX  +80156580
#define DT_A	64	
#define DT_B	76

pDATE caldate(double JD, pDATE date);
double daybeg(double T);
double dt(double T);
double equotime(double T);
double getgmst(void);
double getlmst(void);
double gettd(void);
const char *gettz(void);
double getut(void);
double gmst(double UT);

double jd(pDATE date);
long _jd0(long year);
double jd0(long year);
double *summertime(int year, double dst[2]);
double yearfrac(double T);
double tdb(double T);
double tdt(double T);


/**************/
/* Nuetzliches */
/**************/
pSPHERIC deg2rad(pSPHERIC deg, pSPHERIC rad);
void __eval(int n, int args, const void *tab, void *dat, double scal);
void __eval2(int n, int args, const void *tab, double *dat, double scal);
void __eval3(int n, int args, const void *tab, double *dat, double scal);
void __eval4(int n, int args, const void *tab, void *coor, double scal);
void __peval(int n, int args, const void *tab, pSINCOS dat);
void polyarg(double arg, int idx, int max);
void polyarg2(double arg, double cor, int idx, int max);
double __psum(int n, const void *tab, pSINCOS dat);
void __psum2(int n, const void *tab, pSINCOS dat, void *s, double T, double dT);
pSPHERIC rad2deg(pSPHERIC rad, pSPHERIC deg);
void turnarg(int arg1, int arg2);
void xcharg(int arg1, int arg2);

/* Sonne, Mond und Planeten */
void abersun(BODY body, pCARTESIC pos);
pSINCOS anomaly(double x, double A, pSINCOS ano);
void aps_earth(long z, pAPSIDE aps);
void aps_jupiter(long z, pAPSIDE aps);
void aps_mars(long z, pAPSIDE aps);
void aps_mercury(long z, pAPSIDE aps);
void aps_venus(long z, pAPSIDE aps);
pASPECT aspect_jupiter(long z, pASPECT asp);
long aspect_jupiter_cycle(double T);
pASPECT aspect_mars(long z, pASPECT asp);
long aspect_mars_cycle(double T);
pASPECT aspect_mercury(long z, pASPECT asp);
long aspect_mercury_cycle(double T);
pASPECT aspect_neptune(long z, pASPECT asp);
long aspect_neptune_cycle(double T);
pASPECT aspect_saturn(long z, pASPECT asp);
long aspect_saturn_cycle(double T);
pASPECT aspect_uranus(long z, pASPECT asp);
long aspect_uranus_cycle(double T);
pASPECT aspect_venus(long z, pASPECT asp);
long aspect_venus_cycle(double T);

pCARTESIC kepler(double t, pCARTESIC pos, pcELEMENT elm);
pCARTESIC orbit(double t, pCARTESIC pos);
pCARTESIC position(BODY body, double T, pCARTESIC pos);
/*
 * Rechtwinklige Positionen diverser Genauigkeiten
 */
pCARTESIC barycenter(double t, pCARTESIC pos);
pCARTESIC earthmoon(double t, pCARTESIC pos);
pCARTESIC moonpos(double T, pCARTESIC pos);
pCARTESIC sunpos(double T, pCARTESIC pos);
pCARTESIC _earth(double T, pCARTESIC pos);
pCARTESIC _earth2(double T, pCARTESIC pos);
pCARTESIC _earth3(double T, pCARTESIC pos);
pCARTESIC _jupiter(double T, pCARTESIC pos);
pCARTESIC _jupiter2(double T, pCARTESIC pos);
pCARTESIC _jupiter3(double T, pCARTESIC pos);
pCARTESIC _mars(double T, pCARTESIC pos);
pCARTESIC _mars2(double T, pCARTESIC pos);
pCARTESIC _mars3(double T, pCARTESIC pos);
pCARTESIC _mercury(double T, pCARTESIC pos);
pCARTESIC _mercury2(double T, pCARTESIC pos);
pCARTESIC _mercury3(double T, pCARTESIC pos);
pCARTESIC _moon(double T, pCARTESIC pos);
pCARTESIC _moon2(double T, pCARTESIC pos);
//pCARTESIC _moon3(double T, pCARTESIC pos);
pCARTESIC _neptune(double T, pCARTESIC pos);
pCARTESIC _neptune2(double T, pCARTESIC pos);
pCARTESIC _neptune3(double T, pCARTESIC pos);
pCARTESIC _pluto(double T, pCARTESIC pos);
pCARTESIC _pluto2(double T, pCARTESIC pos);
pCARTESIC _saturn(double T, pCARTESIC pos);
pCARTESIC _saturn2(double T, pCARTESIC pos);
pCARTESIC _saturn3(double T, pCARTESIC pos);
pCARTESIC _sun(double T, pCARTESIC pos);
pCARTESIC _sun2(double T, pCARTESIC pos);
pCARTESIC _sun3(double T, pCARTESIC pos);
pCARTESIC _uranus(double T, pCARTESIC pos);
pCARTESIC _uranus2(double T, pCARTESIC pos);
pCARTESIC _uranus3(double T, pCARTESIC pos);
pCARTESIC _venus(double T, pCARTESIC pos);
pCARTESIC _venus2(double T, pCARTESIC pos);
pCARTESIC _venus3(double T, pCARTESIC pos);
pSTATVEC _earth_bary(double t, pSTATVEC stat);
pSTATVEC __earth_bary(double t, pSTATVEC stat);
pSTATVEC _emb(double t, pSTATVEC stat);
pSTATVEC __emb(double t, pSTATVEC stat);
pSTATVEC _sun_bary(double t, pSTATVEC stat);
pSTATVEC __sun_bary(double t, pSTATVEC stat);
/*
 * Sphaerische Positionen diverser Genauigkeiten
 */
pSPHERIC __earth(double T, pSPHERIC pos);
pSPHERIC __earth2(double T, pSPHERIC pos);
pSPHERIC __earth3(double T, pSPHERIC pos);
pSPHERIC __jupiter(double T, pSPHERIC pos);
pSPHERIC __jupiter2(double T, pSPHERIC pos);
pSPHERIC __jupiter3(double T, pSPHERIC pos);
pSPHERIC __mars(double T, pSPHERIC pos);
pSPHERIC __mars2(double T, pSPHERIC pos);
pSPHERIC __mars3(double T, pSPHERIC pos);
pSPHERIC __mercury(double T, pSPHERIC pos);
pSPHERIC __mercury2(double T, pSPHERIC pos);
pSPHERIC __mercury3(double T, pSPHERIC pos);
pSPHERIC __moon(double T, pSPHERIC pos);
pSPHERIC __moon2(double T, pSPHERIC pos);
//pSPHERIC __moon3(double T, pSPHERIC pos);
pSPHERIC __neptune(double T, pSPHERIC pos);
pSPHERIC __neptune2(double T, pSPHERIC pos);
pSPHERIC __neptune3(double T, pSPHERIC pos);
pSPHERIC __pluto(double T, pSPHERIC pos);
pSPHERIC __pluto2(double T, pSPHERIC pos);
pSPHERIC __saturn(double T, pSPHERIC pos);
pSPHERIC __saturn2(double T, pSPHERIC pos);
pSPHERIC __saturn3(double T, pSPHERIC pos);
pSPHERIC __sun(double T, pSPHERIC pos);
pSPHERIC __sun2(double T, pSPHERIC pos);
pSPHERIC __sun3(double T, pSPHERIC pos);
pSPHERIC __uranus(double T, pSPHERIC pos);
pSPHERIC __uranus2(double T, pSPHERIC pos);
pSPHERIC __uranus3(double T, pSPHERIC pos);
pSPHERIC __venus(double T, pSPHERIC pos);
pSPHERIC __venus2(double T, pSPHERIC pos);
pSPHERIC __venus3(double T, pSPHERIC pos);



double carrington(int c);
double d_moon(double T);
void eclipse(long z, pECLIPSE ecl);
char *ecltype(char *buf, pECLIPSE ecl);
void element(pCARTESIC R, pCARTESIC V, pXELEMENT elm);
double e_sun(double T);
double f_moon(double T);
double k_moon(double T);
double l_moon(double T);
double l0_earth(double T);
double l0_jupiter(double T);
double l0_mars(double T);
double l0_neptune(double T);
double l0_saturn(double T);
double l0_uranus(double T);
double l0_venus(double T);
void magnitude(pPHYSICAL phy);
void moonapside(long z, pAPSIDE apside);
void moondeclmax(long z, pDECLMAX declmax);
void moonnode(long z, pNODE node);
double _moonphase(long z, int k);
pPHASE moonphase(long z, pPHASE phase);
double m_moon(double T);
double m_sun(double T);
void orientation(BODY body, double T, pROTATION rot);
double phasecor(pCARTESIC geo, pCARTESIC hel);
void physical(pPHYSICAL phy);
void riset(SPHERIC equ[3], pRISET rts);
void rotation(pCARTESIC pos, pROTATION rot, pPLANGRAF pla);
void rot_jupiter(double T, pROTATION rot);
void rot_mars(double T, pROTATION rot);
void rot_mercury(double T, pROTATION rot);
void rot_moon(double T, pROTATION rot);
void rot_neptune(double T, pROTATION rot);
void rot_pluto(double T, pROTATION rot);
void rot_saturn(double T, pROTATION rot);
void rot_sun(double T, pROTATION rot);
void rot_uranus(double T, pROTATION rot);
void rot_venus(double T, pROTATION rot);
pSEASON season(double year, pSEASON sea);
double zodiac(double year, ZODIAC z);


void _mnpopup(void);				/* Fehlermeldung im Fenster */


/**********/
/* Makros */
/**********/
#define ASTEROIDS(i)	(&asteroids[(i)-1])
#define DEG		degree
#define HOR		horrad
#define RAD		radian
#define ISGREGOR(y)	((y) * 10000 > calref)		/* Integer! */
#define BESSEL(y)	(((y) - 2000.00127751) * 9.999786414261984E-03)
#define JULIAN(y)	(((y) - 2000.0) *.01)
#define JDC(t)		(JD2000 + (t) * DAY_PER_CY)	/* TD -> JD */
#define JDE(t)		(jd2000 + (t) * daycen)		/* TD -> JD */
#define MJD(t)		(51544.5 + (t) * daycen)	/* Modifiziertes JD */
#define MTD(mjd)	(((mjd) - 51544.5) * cenday)
#define TDC(jd)		(((jd) - JD2000) * CY_PER_DAY)	/* JD -> TD */
#define TDT(jd)		(((jd) - jd2000) * cenday)	/* JD -> TD */
#define YEAR(t)		((t) * 100.0 + 2000.0)
#define LMT(t)		((t) - fun->tim.dt - tzone)	/* TD -> LMT */

#if __cplusplus
}
#endif


#endif		/* ASTRO_H */



