/*
 * CALENDAR is distributed under the GNU GENERAL PUBLIC LICENSE (GPL)
 * Version 2 (June 1991). See the "COPYING" file distributed with this
 * software for more info.
 *
 * Copyright (C) 2000 - 2003 by Michael Schroeder <mdhs@imsch.de>
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "fmath.h"
#include "astro.h"
#include "starcalc.h"
#include "calendar.h"


#define B0	38.8951886398718
#define B1	365.242198780
#define J0	352.25
#define J1	365.25

#define C1	(1.0 / 12.0)		/* 1/12 */
#define C2	(F_PI / 6.0)		/* 2*Pi/12 */


/*
 * Polynom: mittlere Zeiten des Jahreszeitenbeginns (Fruehlingspunkt)
 */
static const double tm[] =
{
   2.15769582477122E-03,
   9.99979121259412E-03,
   1.41519507186858E-12,
  -1.12525667351129E-16,
};


/*
 * Modifizierte Version von __zodiac() (spart ca. 50 kB)
 */
double __zodiac(double T, int z)
{
  SPHERIC pos;
  double t;
  int i = 0;

  do {

    if (i++ > 10)			/* Ueberlauf ? */
      return F_NAN;

    __sun2(T, &pos);		/* maessig genaue Sonnenposition */

	/* in scheinbare ekliptikale Koordinaten umwandeln */

    pos.l -= 9.933744556e-5 / pos.r
           + (8.3386e-05 + T*8.4455e-08) * sin(2.1824 - T*3.37570459367e01)
           + (6.3932e-06 + T*7.7570e-10) * sin(3.5069 + T*1.25666393242e03)
           + (1.1025e-06 + T*9.6963e-11) * sin(1.3375 + T*1.67994182248e04)
           - (9.9969e-07 + T*9.6963e-11) * sin(4.3649 - T*6.75140918733e01)
           - (6.9134e-07 - T*1.7453e-09) * sin(6.2400 + T*6.28301956024e02);

    T += t = 0.001587953 * sin(C2 * z - pos.l);

  } while (fabs(t) > 5 * CY_PER_SEC);	/* Korrektur ueber 5 Sekunden? */ 

  return T;
}


double zodiac(double year, ZODIAC z)
{
  return __zodiac(poly(year + C1 * z, tm), z);
}


/*
 * Mondphasen, p: 0=Neumond,... 2=Vollmond ...
 */
static double get_moonphase(double T, int p)
{
  double t;
  long l = ifloor(T * 1236.853088);	/* # synodischer Umlauf */

  while ((t = _moonphase(l, p)) < T)	/* Zeitpunkt davor ? */
    l++;

  return t;				/* TD der Phase */
}


/*
 * Mondapsiden, i: 0=Perigaeum, 1=Apogaeum
 */
static double get_moonapside(double T, int i)
{
  APSIDE apside;
  double t;
  long l = ifloor(T * 1325.552418) - 1;	/* # anomalistischer Umlauf */

  do {
    moonapside(l++, &apside);

    t = i ? apside.atime: apside.ptime;

  } while (t < T);			/* Zeitpunkt davor ? */

  return t;				/* TD der Apside */
}


/*
 * Monddeklination, i: 0=maximale, 1=minimale Deklination
 */
static double get_moondeclmax(double T, int i)
{
  DECLMAX decl;
  double t;
  long l = ifloor(T * 1336.855226) - 1;	/* # tropischer Umlauf */

  do {
    moondeclmax(l++, &decl);

    t = i ? decl.stime: decl.ntime;

  } while (t < T);			/* Zeitpunkt davor ? */

  return t;				/* TD des Maximums */
}


/*
 * Mondknoten, i: 0=aufsteigender, 1=absteigender Knoten
 */
static double get_moonnode(double T, int i)
{
  NODE node;
  double t;
  long l = ifloor(T * 1342.227826) - 1;	/* # drakonitischer Umlauf */

  do {
    moonnode(l++, &node);

    t = i ? node.des: node.asc;

  } while (t < T);			/* Zeitpunkt davor ? */

  return t;				/* TD des Knotens */
}


/*
 * Finsternis, i: 0=Sonnenfinsternis, 1=Mondfinsternis
 */
static double get_eclipse(double T, int i)
{
  ECLIPSE ecli;
  double t;
  int x;
  long l = ifloor(T * 1236.853088) - 1;	/* # synodischer Umlauf */

  switch (i) {
    case A_SOLAR:
      ecli.e = 0;
      x = 0;
      break;

    case A_LUNAR:
      ecli.e = 1;
      x = 0;
      break;

    default:			/* die naechste Finsternis */
      ecli.e = 0;
      x = 1;
  }

  do {
    do {
      ecli.e ^= x;

      eclipse(l, &ecli);

      l += ecli.e | (x ^ 1);

    } while (ecli.type == 0);		/* gefunden ? */

    t = ecli.tmax;

  } while (t < T);			/* Zeitpunkt davor ? */

  return t;				/* TD der Finsternis */
}


/*
 * Tierkreis, z: 0=Widder,... 6=Waage ...
 */
static double get_zodiac(double T, int z)
{
  double t;
  long y = ifloor(T * 100.0021384);	/* # Umlauf (Jahr - 2000) */

  while ((t = zodiac(y, z)) < T)	/* Zeitpunkt davor ? */
    y++;

  return t;				/* TD des Tierkreiszeichens */
}


/*
 * Tierkreis,  (naechstes Zeichen)
 */
static double get_next_zodiac(double T)
{
  double t;
  long y, z;

  y = idiv2(ifloor(T * 100.0021384 * 12) - 2, 12, &z);	/* Jahr und Zeichen */

  while ((t = zodiac(y, z)) < T)	/* Zeitpunkt davor ? */
    y += idiv2(z + 1, 12, &z);		/* naechstes Zeichen */

  return t;
}


/*
 * Jahreszeiten, s: 0=Fruehling,... 2=Herbst ...
 */
static double get_season(double T, int s)
{
  return get_zodiac(T, 3 * s);
}


/*
 * Merkur, z: Aspekte  
 */
static double get_mercury(double T, int z)
{
  ASPECT asp;
  double t;
  long l = aspect_mercury_cycle(T);

  do
    aspect_mercury(l++, &asp);
  while ((t = GETASPECT(z, &asp)) < T);

  return t;
}


/*
 * Venus, z: Aspekte  
 */
static double get_venus(double T, int z)
{
  ASPECT asp;
  double t;
  long l = aspect_venus_cycle(T);

  do
    aspect_venus(l++, &asp);
  while ((t = GETASPECT(z, &asp)) < T);

  return t;
}


/*
 * Mars, z: Aspekte  
 */
static double get_mars(double T, int z)
{
  ASPECT asp;
  double t;
  long l = aspect_mars_cycle(T);

  do
    aspect_mars(l++, &asp);
  while ((t = GETASPECT(z, &asp)) < T);

  return t;
}


/*
 * Jupiter, z: Aspekte  
 */
static double get_jupiter(double T, int z)
{
  ASPECT asp;
  double t;
  long l = aspect_jupiter_cycle(T);

  do
    aspect_jupiter(l++, &asp);
  while ((t = GETASPECT(z, &asp)) < T);

  return t;
}


/*
 * Jupiter, z: Aspekte  
 */
static double get_saturn(double T, int z)
{
  ASPECT asp;
  double t;
  long l = aspect_saturn_cycle(T);

  do
    aspect_saturn(l++, &asp);
  while ((t = GETASPECT(z, &asp)) < T);

  return t;
}


/*
 * Uranus, z: Aspekte  
 */
static double get_uranus(double T, int z)
{
  ASPECT asp;
  double t;
  long l = aspect_uranus_cycle(T);

  do
    aspect_uranus(l++, &asp);
  while ((t = GETASPECT(z, &asp)) < T);

  return t;
}


/*
 * Neptun, z: Aspekte  
 */
static double get_neptune(double T, int z)
{
  ASPECT asp;
  double t;
  long l = aspect_neptune_cycle(T);

  do
    aspect_neptune(l++, &asp);
  while ((t = GETASPECT(z, &asp)) < T);

  return t;
}


double astronomical(double jd, int type, int index)
{
  double d, t = JD2TD(jd);

  t += d = dt(t);		/* t in dynamischer Zeit (TD) */

  switch (type & 0xffff) {

    case A_JULIAN:		/* Julianischer Jahresanfang */
      if (type & 0x10000)
        jd += 182;
      return floor((jd - J0) / J1) * J1 + J0;

    case A_BESSEL:		/* Besselscher Jahresanfang */
      if (type & 0x10000)
        jd += 182;
      return floor((jd - B0) / B1) * B1 + B0;

    case A_MOONPHASE:
      t = get_moonphase(t, index);
      break;

    case A_MOONAPSIDE:
      t = get_moonapside(t, index);
      break;

    case A_MOONDECLMAX:
      t = get_moondeclmax(t, index);
      break;

    case A_MOONNODE:
      t = get_moonnode(t, index);
      break;

    case A_ECLIPSE:
      t = get_eclipse(t, index);
      break;

    case A_SEASON:
      t = get_season(t, index);
      break;

    case A_ZODIAC:
      t = get_zodiac(t, index);
      break;

    case A_NEXT_ZODIAC:
      t = get_next_zodiac(t);
      break;

    case A_MERCURY:
      t = get_mercury(t, index);
      break;

    case A_VENUS:
      t = get_venus(t, index);
      break;

    case A_MARS:
      t = get_mars(t, index);
      break;

    case A_JUPITER:
      t = get_jupiter(t, index);
      break;

    case A_SATURN:
      t = get_saturn(t, index);
      break;

    case A_URANUS:
      t = get_uranus(t, index);
      break;

    case A_NEPTUNE:
      t = get_neptune(t, index);
      break;

    default:
      t = INVALID;
  }

  return TD2JD(t - d - tzone);   	/* JD in Ortszeit (LMT) */
}
