/*
 * CALENDAR is distributed under the GNU GENERAL PUBLIC LICENSE (GPL)
 * Version 2 (June 1991). See the "COPYING" file distributed with this
 * software for more info.
 *
 * Copyright (C) 2000 - 2003 by Michael Schroeder <mdhs@imsch.de>
 */


/*
 * Kodierung von Kalenderdaten:
 *
 *	Base    = Basisereignis (Ursprung der Berechnung, Nullpunkt) 
 *	Offset  = Abstand des Termins vom Basisereignis
 *	Phase   = Korrekturglied fuer Termine mit konstantem Wochentag
 *
 *	EVT(Base, Offset)		fuer feste und bewegliche Termine
 *	EVT2(Base, Offset, Phase)	fuer Termine mit konstantem Wochentag
 *
 *   In Kalendern deren Schalttage nicht am Jahresende angefuegt werden,
 *   muessen Termine nach dem Schalttag vom folgenden Jahr aus rueckwaerts
 *   gezaehlt werden:
 *	(Base = {gregorian_reverse, julian_reverse...} und negativer Offset) 
 *
 *
 *   Berechnung von Phase und Offset fuer 'N-ter Wochentag D im Monat M':
 *
 *	N = n-ter Wochentag (1 = erster, ... 0 = letzter, -1 = vorletzter ...)
 *	D = Wochentag (0=So, 1=Mo, 2=Di ... 6=Sa)
 *	M = Monat
 *
 *	Monatslaenge[] = {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}
 *
 *	Offset = (275*M)/9-367*((M+7)/10)+7*N-31+(N <= 0)*Monatslaenge[M]
 *	Phase  = (6+(372+Offset)%7-D)%7+1
 *
 *   Beispiele:
 *
 *   - 7 Tage vor Neujahr (1. Weihnachtstag) = EVT(B_REVERSE, -7)
 *
 *	Base = B_REVERSE; Offset = -7 
 *
 *   - 46 Tage vor Ostern (Aschermittwoch) = EVT(B_EASTER, -46)
 *
 *	Base = B_EASTER; Offset = -46 
 *
 *   - 49 Tage nach Ostern (Pfingsten) = EVT(B_EASTER, +49)
 *
 *	Base = B_EASTER; Offset = +49 
 *
 *   - 1. Sonntag im Oktober (Erntedank) = EVT2(B_REVERSE, -86, 6)
 *
 *	N = 1; D = 0; M = 10   ==>  Base = B_REVERSE; Offset = -86; Phase = 6
 *
 *   - 4. Donnerstag im November (Thanksgiving) = EVT2(B_REVERSE, -34, 5)
 *
 *	N = 4; D = 4; M = 11   ==>  Base = B_REVERSE; Offset = -34; Phase = 5
 *
 *   - fuenftletzter Sonntag im Jahr (1. Advent) = EVT2(B_REVERSE, -29, 7)
 *
 *	N = -4; D = 0; M = 12  ==>  Base = B_REVERSE; Offset = -29; Phase = 7
 *
 *   - 3. Montag im Februar (President's Day) = EVT2(B_JD0, +51, 2)
 *
 *	N = 3; D = 1; M = 2  ==>  Base = B_JD0; Offset = +51; Phase = 2
 */
#define EVT(b,o)	EVT2(b,o,0)
#define EVT2(b,o,p)	(void*)(((b) << 24) | ((unsigned char)(p) << 16) | (unsigned short)(o))
#define BASE(e)		((e) >> 24)
#define PHASE(e)	((signed char)((e) >> 16))
#define OFFSET(e)	((signed short)(e))
#define AVOID(e)	(PHASE(e) & 0x0f)
#define SHIFT(e)	(PHASE(e) >> 4)
#define MOVE(a,s)	((unsigned char)(((s) << 4) | (((a+6)%7) & 0x0f)))

#define MAX_LOOP	(1 << 16)	/*  Suchtiefe: max. 2^16 Zyklen */

//#define DEBUG 
#ifdef DEBUG
# define DBGMSG(msg...)	fprintf(stderr, ADDIN ": " ##msg)  /* GNU extension */
#else
# define DBGMSG(msg...)
#endif


/*
 * Zeichen zu einer Nummer zusammenbasteln (little endian!)
 */
#define MKNUM(a,b)	((((unsigned char)b)<<8)|((unsigned char)a))

/*
 * Zeichen ausblenden und in Grossbuchstaben umwandeln (auch ANSI-Umlaute)
 */
#define GET1(c)		(*(unsigned char*)(c) & 0xdf)
#define GET2(c)		(*(unsigned short*)(c) & 0xdfdf)

#ifdef __EMX__
# define SKIPSPACE(s)	while (isspace((int)*s)) s++
#else
# define SKIPSPACE(s)	while (isspace(*s)) s++
#endif


#define INVALID		F_QNAN			/* Err:503 (ungueltig) */

#define ENTRIES(n)	(sizeof(n)/sizeof(n[0]))


typedef enum
{
  FUNC_ARMENIAN = 0,
  FUNC_ASTRONOMY,
  FUNC_BUDDHIST,
  FUNC_BYZANTINE,
  FUNC_CALENDAR,
  FUNC_CHINESE,
  FUNC_CHRONOLOGY,
  FUNC_COPTIC,
  FUNC_EGYPTIAN,
  FUNC_EVENT,
  FUNC_ETHIOPIC,
  FUNC_FRENCH,
  FUNC_GENGOU,
  FUNC_GNOMON,
  FUNC_GREGORIAN,
  FUNC_HEBREW,
  FUNC_HINDU,
  FUNC_HINDU2,
  FUNC_HOLIDAY,
  FUNC_HOLIDAY2,
  FUNC_INDIAN,
  FUNC_ISLAMIC,
  FUNC_ISO8601,
  FUNC_JULIAN,
  FUNC_LEAPYEAR,
  FUNC_LEAPYEAR2,
  FUNC_MAYAN,
  FUNC_PERSIAN,
  FUNC_ROMAN,
  FUNC_SELEUCID,
  FUNC_SYRIAN,
  FUNC_TAIWANESE,
  FUNC_WHEN,
  FUNC_WHEN2,
  FUNC_YAZDEGERD,

  COUNT_OF_FUNCS,	/* dieser Wert ist die Anzahl aller Funktionen */ 
} FUNCS;


typedef enum
{
  DESCR( ARMENIAN ),
  DESCR( ASTRONOMY ),
  DESCR( CALENDAR ),
  DESCR( BUDDHIST ),
  DESCR( BYZANTINE ),
  DESCR( CHINESE ),
  DESCR( CHRONOLOGY ),
  DESCR( COPTIC ),
  DESCR( EGYPTIAN ),
  DESCR( EVENT ),
  DESCR( ETHIOPIC ),
  DESCR( FRENCH ),
  DESCR( GENGOU ),
  DESCR( GNOMON ),
  DESCR( GREGORIAN ),
  DESCR( HEBREW ),
  DESCR( HINDU ),
  DESCR( HINDU2 ),
  DESCR( HOLIDAY ),
  DESCR( HOLIDAY2 ),
  DESCR( INDIAN ),
  DESCR( ISLAMIC ),
  DESCR( ISO8601 ),
  DESCR( JULIAN ),
  DESCR( MAYAN ),
  DESCR( LEAPYEAR ),
  DESCR( LEAPYEAR2 ),
  DESCR( PERSIAN ),
  DESCR( ROMAN ),
  DESCR( SELEUCID ),
  DESCR( SYRIAN ),
  DESCR( TAIWANESE ),
  DESCR( WHEN ),
  DESCR( WHEN2 ),
  DESCR( YAZDEGERD ),

  DESCR( CALTYPE ),
  DESCR( CALTYPE1 ),
  DESCR( CALTYPE2 ),
  DESCR( CALCYCLE ),
  DESCR( CALMODE ),
  DESCR( CELESTIAL ),
  DESCR( CYCLE ),
  DESCR( DATE ),
  DESCR( DAY ),
  DESCR( DAYNAME ),
  DESCR( EPOCH ),
  DESCR( FIXPOINT ),
  DESCR( GENGOU_ERA ),
  DESCR( HOLIDAYS ),
  DESCR( HOLIDAYS1 ),
  DESCR( HOLIDAYS2 ),
  DESCR( LEAPMONTH ),
  DESCR( MONTH ),
  DESCR( MONTHNAME ),
  DESCR( ORDINAL ),
  DESCR( START ),
  DESCR( WEEK ),
  DESCR( YEAR ),
  DESCR( BAKTUN ),
  DESCR( KATUN ),
  DESCR( TUN ),
  DESCR( UINAL ),
  DESCR( KIN ),
} STUFF;


/*
 * Ausgangspunkte von Ereignissen
 */
typedef enum
{
  B_ASTRONOMICAL = 0,
  B_JD0,		/* Basis: Neujahr */
  B_EASTER,		/* Basis: Ostern */
  B_REVERSE,		/* Basis: Neujahr des folgenden Jahres */
  B_JULIAN_JD0,
  B_JULIAN_EASTER,
  B_JULIAN_REVERSE,

  B_ARMENIAN_JD0,
  B_BUDDHIST_JD0,
  B_BYZANTINE_JD0,
  B_CHINESE_JD0,
  B_CHINESE_DONGZHI,
  B_CHINESE_XIAZHI,
  B_CHINESE_CHUSHU,
  B_COPTIC_JD0,
  B_COPTIC_EASTER,
  B_EGYPTIAN_JD0,
  B_ETHIOPIC_JD0,
  B_ETHIOPIC_EASTER,
  B_FRENCH_JD0,
  B_GENGOU_JD0,
  B_HEBREW_JD0,
  B_HEBREW_KISLEV,
  B_HEBREW_TEVET,
  B_HEBREW_REVERSE,
  B_HINDU_JD0,
  B_HINDU2_JD0,
  B_INDIAN_JD0,
  B_ISLAMIC_JD0,
  B_ISO8601_JD0,
  B_MAYAN_JD0,
  B_MAYAN_HAAB,
  B_MAYAN_TZOLKIN,
  B_PERSIAN_JD0,
  B_SELEUCID_JD0,
  B_SYRIAN_JD0,
  B_TAIWANESE_JD0,
  B_YAZDEGERD_JD0,
} BASEEVENTS;


typedef enum
{
  MONDAY,
  TUESDAY,
  WEDNESDAY,
  THURSDAY,
  FRIDAY,
  SATURDAY,
  SUNDAY,
} DAYOFWEEK;


typedef enum
{
  C_EPACT,
  C_INDICTION,
  C_REGENT,
  C_AUREUS,
  C_SUNCIRC,
  C_DOMINICAL,
} CHRONOLOGY;


typedef enum
{
  A_JULIAN = 1,
  A_BESSEL,
  A_MOONPHASE,
  A_MOONAPSIDE,
  A_MOONDECLMAX,
  A_MOONNODE,
  A_ECLIPSE,
  A_SEASON,
  A_ZODIAC,
  A_NEXT_ZODIAC,
  A_MERCURY,
  A_VENUS,
  A_MARS,
  A_JUPITER,
  A_SATURN,
  A_URANUS,
  A_NEPTUNE,
} ASTRONOMY;

typedef enum
{
  A_SOLAR,
  A_LUNAR,
  A_ALL,
} ECLIPSETYPE;


typedef const char *STRING, *STRINGS[];
typedef void *POINTER, *POINTERS[];
typedef double (*EVENT)(long);
typedef pDATE (*CALDATE)(double, pDATE);
typedef int (*LEAPYEAR)(long);
typedef int (*SPRINTF)(char *, double, int);


#define ITEMS(i...)	(STRINGS){i}
#define FUNCS(f...)	(POINTERS){f}
#define SUBFUNCS(f, i1, i2...)	{sizeof(i1) / sizeof(STRING*), f, i1, i2}


typedef struct
{
  int      count;		/* Anzahl der Funktionen */
  POINTER *funcs;		/* Zeiger auf Funktionen */
  const STRING *names;		/* englische Bezeichnung */
  const STRING *names_de;	/* deutsche Bezeichnung */
/* const STRINGS *names_fr */	/* zur Spracherweiterung... */
} FUNCLIST;


/*
 * astronom.c
 */
double astronomical(double, int, int);

/*
 * calfunc.c
 */
void init_addin(int);
SO_QUIRKSCALL(SC_armenian, double *, double *, char *, double *);
SO_CALLBACK(SC_astronomy, double *r, double *);
SO_QUIRKSCALL(SC_buddhist, double *, double *, char *, double *);
SO_QUIRKSCALL(SC_byzantine, double *, double *, char *, double *);
SO_QUIRKSCALL(SC_calendar, char *, char *, double *, double *);
SO_QUIRKSCALL(SC_chinese, double *, double *, double *, char *, double *, double *);
SO_CALLBACK(SC_chronology, char *, char *, double *);
SO_QUIRKSCALL(SC_coptic, double *, double *, char *, double *);
SO_QUIRKSCALL(SC_egyptian, double *, double *, char *, double *);
SO_QUIRKSCALL(SC_ethiopic, double *, double *, char *, double *);
SO_QUIRKSCALL(SC_event, double *, double *, char *, char *, double *);
SO_QUIRKSCALL(SC_french, double *, double *, char *, double *);
SO_QUIRKSCALL(SC_gengou, double *, char *, double *, char *, double *);
SO_CALLBACK(SC_gnomon, double *, double *);
SO_QUIRKSCALL(SC_gregorian, double *, double *, char *, double *);
SO_QUIRKSCALL(SC_hebrew, double *, double *, char *, double *);
SO_QUIRKSCALL(SC_hindu, double *, double *, char *, double *, double *);
SO_QUIRKSCALL(SC_hindu2, double *, double *, char *, double *);
SO_QUIRKSCALL(SC_holiday, double *, char *, char *, char *);
SO_QUIRKSCALL(SC_holiday2, double *, char *, char *, double *);
SO_QUIRKSCALL(SC_indian, double *, double *, char *, double *);
SO_QUIRKSCALL(SC_islamic, double *, double *, char *, double *);
SO_QUIRKSCALL(SC_iso8601, double *, double *, double *, char *);
SO_QUIRKSCALL(SC_julian, double *, double *, char *, double *);
SO_QUIRKSCALL(SC_mayan, double *, double *, double*, double*, double*, double*);
SO_QUIRKSCALL(SC_latin, double *, double *, char *, char *, char *);
SO_CALLBACK(SC_leapyear, double *, char *, char *);
SO_CALLBACK(SC_leapyear2, double *, char *, double *);
SO_QUIRKSCALL(SC_persian, double *, double *, char *, double *);
SO_QUIRKSCALL(SC_seleucid, double *, double *, char *, double *);
SO_QUIRKSCALL(SC_taiwanese, double *, double *, char *, double *);
SO_QUIRKSCALL(SC_when, double *, char *, char *, char *, double *, char *, double *);
SO_QUIRKSCALL(SC_when2, double *, char *, char *, char *, char *, double *);
SO_QUIRKSCALL(SC_yazdegerd, double *, double *, char *, double *);

/*
 * main.c
 */
extern unsigned long _max_loop;
SO_CALLBACK(GetFunctionCount, USHORT *);
SO_QUIRKSCALL(GetFunctionData, USHORT *, char *, USHORT *, ParamType *, char *);
SO_QUIRKSCALL(GetParameterDescription, USHORT *, USHORT *, char *, char *);

/*
 * string.c
 */
void *scan(const char *, const FUNCLIST *);
int strchk(const char *s1, const char *s2, int n);

