/*
 * CALENDAR is distributed under the GNU GENERAL PUBLIC LICENSE (GPL)
 * Version 2 (June 1991). See the "COPYING" file distributed with this
 * software for more info.
 *
 * Copyright (C) 2000 - 2003 by Michael Schroeder <mdhs@imsch.de>
 */

/*
 * Hints:
 *	Wenn in Suchlisten die lexikalische aufsteigende Reihenfolge nicht
 *	eingehalten wird, dann werden Begriffe die eine Untermenge der
 *	vorangegangenen Bregriffr darstellen nicht mehr gefunden.
 *
 *	Bsp.: "Hindu" wird _nach_ "Hindu2" in der Liste nicht gefunden	  
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "fmath.h"
#include "astro.h"
#include "calend.h"
#include "starcalc.h"
#include "calendar.h"


#define iso8601_holidays	gregorian_holidays
#define gregorian_monnames	julian_monnames
#define roman_holidays		julian_holidays


static int astronomical_sprintf(char *, double, int);
static int calendar_sprintf(char *, double, int);
static double coptic_easter(long);
static double ethiopic_easter(long);
static double gregorian_reverse(long);
static double hebrew_reverse(long);
static double julian_reverse(long);
static double next_event(double, long, void *);


static const STRING calendar_names_en[] =
{
  "armenian",
  "astronomical",
  "buddhist",
  "byzantine",
  "chinese",
  "coptic",
  "egyptian",
  "ethiopic",
  "french",
  "gengou",
  "gregorian",
  "hebrew",
  "hindu",
  "hindu2",
  "indian",
  "islamic",
  "iso8601",
  "julian",
  "latin",
  "mayan",
  "persian",
  "seleucid",
  "syrian",
  "taiwanese",
  "yazdegerd",
};

static STRING calendar_names_de[] =
{
  "armenisch",
  "astronomisch",
  "buddhistisch",
  "byzantinisch",
  "chinesisch",
  "koptisch",
   ae"gyptisch",
   ae"thiopisch",
  "franz"oe"sisch",
  "gengou",
  "gregorianisch",
  "j"ue"disch",
  "hindu",
  "hindu2",
  "indisch",
  "islamisch",
  "iso8601",
  "julianisch",
  "lateinisch",
  "maya",
  "persisch",
  "seleukiden",
  "syrisch",
  "taiwanisch",
  "jezdegerd",
};


static const FUNCLIST day_names =
{
  7,
  NULL,
  dayname_en,
  dayname_de,
};


static const FUNCLIST calendar = SUBFUNCS
(
  FUNCS(armenian_sprintf, astronomical_sprintf,
	buddhist_sprintf, byzantine_sprintf,
	chinese_sprintf, coptic_sprintf,
        egyptian_sprintf, ethiopic_sprintf,
	french_sprintf, gengou_sprintf,
        gregorian_sprintf, hebrew_sprintf,
	hindu_sprintf, hindu2_sprintf,
	indian_sprintf, islamic_sprintf,
  	iso_sprintf, julian_sprintf,
	roman_sprintf, mayan_sprintf,
	persian_sprintf, seleucid_sprintf,
	syrian_sprintf,
	taiwanese_sprintf, yazdegerd_sprintf,
	),
  calendar_names_en,
  calendar_names_de,
);


static const FUNCLIST jd2other = SUBFUNCS
(
  FUNCS(armenian_caldate, caldate,
	buddhist_caldate, byzantine_caldate,
	chinese_caldate, coptic_caldate,
        egyptian_caldate, ethiopic_caldate,
	french_caldate, gengou_caldate,
        gregorian_caldate, hebrew_caldate,
	hindu_caldate, hindu2_caldate,
	indian_caldate, islamic_caldate,
	iso_caldate, julian_caldate,
	roman_caldate, mayan_caldate,
	persian_caldate, seleucid_caldate,
	syrian_caldate,
	taiwanese_caldate, yazdegerd_caldate,
	),
  calendar_names_en,
  calendar_names_de,
);


static const FUNCLIST leapyear = SUBFUNCS
(
  FUNCS(armenian_isleap, isleap,
	buddhist_isleap, byzantine_isleap,
	chinese_isleap, coptic_isleap,
        egyptian_isleap, ethiopic_isleap,
	french_isleap, gengou_isleap,
        gregorian_isleap, hebrew_isleap,
	hindu_isleap, hindu2_isleap,
	indian_isleap, islamic_isleap,
	iso_isleap, julian_isleap,
	roman_isleap, mayan_isleap,
	persian_isleap, seleucid_isleap,
	syrian_isleap,
	taiwanese_isleap, yazdegerd_isleap,
	),
  calendar_names_en,
  calendar_names_de,
);


static const FUNCLIST astronomy = SUBFUNCS
(
  FUNCS(EVT2(0, A_JULIAN, 0), EVT2(0, A_JULIAN, 0), EVT2(0, A_BESSEL, 0),   
	EVT2(0, A_MOONPHASE, 0), EVT2(0, A_MOONPHASE, 1),
	EVT2(0, A_MOONPHASE, 2), EVT2(0, A_MOONPHASE, 3),
	EVT2(0, A_MOONAPSIDE, 0), EVT2(0, A_MOONAPSIDE, 1),
	EVT2(0, A_MOONDECLMAX, 0), EVT2(0, A_MOONDECLMAX, 1),
	EVT2(0, A_MOONNODE, 0), EVT2(0, A_MOONNODE, 1),
	EVT2(0, A_ECLIPSE, A_SOLAR), EVT2(0, A_ECLIPSE, A_LUNAR),
	EVT2(0, A_ECLIPSE, A_ALL),
	EVT2(0, A_SEASON, 0), EVT2(0, A_SEASON, 1),
	EVT2(0, A_SEASON, 2), EVT2(0, A_SEASON, 3),
	EVT2(0, A_ZODIAC, 0), EVT2(0, A_ZODIAC, 1), EVT2(0, A_ZODIAC, 2),
	EVT2(0, A_ZODIAC, 3), EVT2(0, A_ZODIAC, 4), EVT2(0, A_ZODIAC, 5),
	EVT2(0, A_ZODIAC, 6), EVT2(0, A_ZODIAC, 7), EVT2(0, A_ZODIAC, 8),
	EVT2(0, A_ZODIAC, 9), EVT2(0, A_ZODIAC, 10), EVT2(0, A_ZODIAC, 11),
	EVT2(0, A_NEXT_ZODIAC, 0),
	EVT2(0, A_MERCURY, A_CONJUNCTION_INFERIOR), EVT2(0, A_MERCURY, A_CONJUNCTION_SUPERIOR),
	EVT2(0, A_MERCURY, A_ELONGATION_EAST), EVT2(0, A_MERCURY, A_ELONGATION_WEST),
	EVT2(0, A_VENUS, A_CONJUNCTION_INFERIOR), EVT2(0, A_VENUS, A_CONJUNCTION_SUPERIOR),
	EVT2(0, A_VENUS, A_ELONGATION_EAST), EVT2(0, A_VENUS, A_ELONGATION_WEST),
	EVT2(0, A_MARS, A_OPPOSITION), EVT2(0, A_MARS, A_CONJUNCTION),
	EVT2(0, A_JUPITER, A_OPPOSITION), EVT2(0, A_JUPITER, A_CONJUNCTION),
	EVT2(0, A_SATURN, A_OPPOSITION), EVT2(0, A_SATURN, A_CONJUNCTION),
	EVT2(0, A_URANUS, A_OPPOSITION), EVT2(0, A_URANUS, A_CONJUNCTION),
	EVT2(0, A_NEPTUNE, A_OPPOSITION), EVT2(0, A_NEPTUNE, A_CONJUNCTION)
	),
  ITEMS("New Year's Day", "Julian Year", "Besselian Year",
	"New Moon", "First Quarter",
	"Full Moon", "Last Quarter",
	"Perigee", "Apogee",
	"North Point", "South Point",
	"Ascending Node", "Descending Node",
	"Solar Eclipse", "Lunar Eclipse",
	"Eclipse",
	"Spring", "Summer",
	"Autumn", "Winter",
	"Aries", "Taurus", "Gemini",
	"Cancer", "Leo", "Virgo",
	"Libra", "Scorpio", "Sagittarius",
	"Capricorn", "Aquarius", "Pisces",
	"Zodiac",
	"Mercury Inferior Conjunction", "Mercury Superior Conjunction",
	"Mercury East Elongation", "Mercury West Elongation",
	"Venus Inferior Conjunction", "Venus Superior Conjunction",
	"Venus East Elongation", "Venus West Elongation",
	"Mars Opposition", "Mars Conjunction",
	"Jupiter Opposition", "Jupiter Conjunction",
	"Saturn Opposition", "Saturn Conjunction",
	"Uranus Opposition", "Uranus Conjunction",
	"Neptune Opposition", "Neptune Conjunction",
	),
  ITEMS("Neujahr", "Julianisches Jahr", "Besselsches Jahr",
	"Neumond", "Erstes Viertel",
	"Vollmond", "Letztes Viertel",
	"Perig"ae"um", "Apog"ae"um",
	"Nordpunkt", "S"ue"dpunkt",
	"Aufsteigender Knoten","Absteigender Knoten",
	"Sonnenfinsternis", "Mondfinsternis",
	"Finsternis",
	"Fr"ue"hlingsanfang", "Sommeranfang",
	"Herbstanfang", "Winteranfang",
	"Widder", "Stier", "Zwillinge",
	"Krebs", "L"oe"we", "Jungfrau",
	"Waage", "Skorpion", "Sch"ue"tze",
	"Steinbock", "Wassermann", "Fische",
	"Sternzeichen",
	"Merkur Untere Konjunktion", "Merkur Obere Konjunktion",
	"Merkur "OE"stliche Elongation", "Merkur Westliche Elongation",
	"Venus Untere Konjunktion", "Venus Obere Konjunktion",
	"Venus "OE"stliche Elongation", "Venus Westliche Elongation",
	"Mars Opposition", "Mars Konjunktion",
	"Jupiter Opposition", "Jupiter Konjunktion",
	"Saturn Opposition", "Saturn Konjunktion",
	"Uranus Opposition", "Uranus Konjunktion",
	"Neptun Opposition", "Neptun Konjunktion"
	),
);


static const FUNCLIST chronology = SUBFUNCS
(
  NULL,
  ITEMS("Epact",
	"Indiction",
	"Regent",
	"Golden Number",
        "Solar Cycle",
	"Dominical Letter"
	),
  ITEMS("Epakte",
	"Indiktion",
	"Jahresregent",
	"Goldene Zahl",
	"Sonnenzirkel",
	"Sonntagsbuchstabe"
	),
);


static const FUNCLIST armenian_holidays = SUBFUNCS
(
  FUNCS(EVT(B_ARMENIAN_JD0, 0)),
  ITEMS("New Year's Day"),
  ITEMS("Neujahr"),
);


static const FUNCLIST buddhist_holidays = SUBFUNCS
(
  FUNCS(EVT(B_BUDDHIST_JD0, 0)),
  ITEMS("New Year's Day"),
  ITEMS("Neujahr"),
);


static const FUNCLIST byzantine_holidays = SUBFUNCS
(
  FUNCS(EVT(B_BYZANTINE_JD0, 0)),
  ITEMS("New Year's Day"),
  ITEMS("Neujahr"),
);


static const FUNCLIST chinese_holidays = SUBFUNCS
(
  FUNCS(EVT(B_CHINESE_JD0, 0),
	EVT(B_CHINESE_JD0, 14),
        EVT(B_CHINESE_XIAZHI, 4),
	EVT(B_CHINESE_CHUSHU, 6),
	EVT(B_CHINESE_CHUSHU, 14),
	EVT(B_CHINESE_DONGZHI, 0),
	),
  ITEMS("New Year's Day",
	"Lantern Festival",
        "Dragon Festival",
	"Valentine's Day",
	"Hungry Ghosts",
	"Dongzhi",
	),
  ITEMS("Neujahr",
	"Laternenfest",
        "Drachenfest",
	"Valentinstag",
	"Hungrige Geister",
	"Dongzhi",
	),
);


static const FUNCLIST coptic_holidays = SUBFUNCS
(
  FUNCS(EVT(B_COPTIC_JD0, 0),
  	EVT(B_COPTIC_JD0, 16),
  	EVT(B_COPTIC_JD0, 118),
  	EVT(B_COPTIC_JD0, 125),
	EVT(B_COPTIC_JD0, 130),
  	EVT(B_COPTIC_JD0, 209),
	EVT(B_COPTIC_EASTER, 0),
  	EVT(B_COPTIC_JD0, 342),
	),
  ITEMS("New Year's Day",
	"Elevation",
	"Christmas",
	"Circumcision",
	"Epiphany",
	"Annunciation",
	"Easter",
	"Transfiguration",
	),
  ITEMS("Neujahr",
	"Kreuzerh"oe"hung",
	"Weihnachten",
	"Beschneidung",
	"Epiphanias",
	"Verk"ue"ndigung",
	"Ostern",
	"Verkl"ae"rung",
	),
);


static const FUNCLIST egyptian_holidays = SUBFUNCS
(
  FUNCS(EVT(B_EGYPTIAN_JD0, 0)),
  ITEMS("New Year's Day"),
  ITEMS("Neujahr"),
);


static const FUNCLIST ethiopic_holidays = SUBFUNCS
(
  FUNCS(EVT(B_ETHIOPIC_JD0, 0),
  	EVT(B_ETHIOPIC_JD0, 16),
  	EVT(B_ETHIOPIC_JD0, 118),
  	EVT(B_ETHIOPIC_JD0, 125),
	EVT(B_ETHIOPIC_JD0, 130),
  	EVT(B_ETHIOPIC_JD0, 209),
	EVT(B_ETHIOPIC_EASTER, 0),
  	EVT(B_ETHIOPIC_JD0, 342),
	),
  ITEMS("New Year's Day",
	"Elevation",
	"Christmas",
	"Circumcision",
	"Epiphany",
	"Annunciation",
	"Easter",
	"Transfiguration",
	),
  ITEMS("Neujahr",
	"Kreuzerh"oe"hung",
	"Weihnachten",
	"Beschneidung",
	"Epiphanias",
	"Verk"ue"ndigung",
	"Ostern",
	"Verkl"ae"rung",
	),

);


static const FUNCLIST french_holidays = SUBFUNCS
(
  FUNCS(EVT(B_FRENCH_JD0, 0)),
  ITEMS("New Year's Day"),
  ITEMS("Neujahr"),
);


static const FUNCLIST gengou_holidays = SUBFUNCS
(
  FUNCS(EVT(B_GENGOU_JD0, 0)),
  ITEMS("New Year's Day"),
  ITEMS("Neujahr"),
);


static const FUNCLIST gregorian_holidays = SUBFUNCS
(
  FUNCS(EVT(B_JD0, 0),
	EVT(B_JD0, 5),
	EVT(B_EASTER, -63),
	EVT(B_EASTER, -56),
	EVT(B_EASTER, -48),
	EVT(B_EASTER, -47), 
        EVT(B_EASTER, -46),
	EVT(B_REVERSE, -282),
	EVT(B_EASTER, -14),
	EVT(B_EASTER, -7),
	EVT(B_EASTER, -3),
	EVT(B_EASTER, -2),
	EVT(B_EASTER, 0),		/* Easter */
	EVT(B_EASTER, 0),
	EVT(B_EASTER, 1),
	EVT(B_EASTER, +35),
	EVT(B_EASTER, +39),
	EVT(B_EASTER, +49),
	EVT(B_EASTER, +49),
	EVT(B_EASTER, +50),
	EVT(B_EASTER, +56),
	EVT(B_EASTER, +60),
	EVT(B_REVERSE, -139),
	EVT(B_REVERSE, -62),
	EVT(B_REVERSE, -61),
	EVT2(B_REVERSE, -40, 7),
	EVT2(B_REVERSE, -36, 7),
	EVT2(B_REVERSE, -29, 7),
	EVT(B_REVERSE, -24),
	EVT(B_REVERSE, -7),		/* Christmas */
	EVT(B_REVERSE, -153),
	EVT(B_REVERSE, -90),
	EVT(B_REVERSE, -67),
	EVT(B_REVERSE, -146),
	EVT2(B_REVERSE, -232, 7),
	EVT2(B_REVERSE, -86, 6),
	EVT(B_JD0, 44),
	EVT(B_REVERSE, -290),
	EVT2(B_JD0, 51, 2),
	EVT2(B_REVERSE, -255, 1),
	EVT2(B_REVERSE, -215, 2),
	EVT2(B_REVERSE, -194, 3),
	EVT(B_REVERSE, -201),
	EVT(B_REVERSE, -181),
	EVT2(B_REVERSE, -116, 3),
	EVT2(B_REVERSE, -79, 5),
	EVT(B_REVERSE, -62),
	EVT2(B_REVERSE, -54, 1),
        EVT(B_REVERSE, -51),
	EVT2(B_REVERSE, -34, 5),
	),
  ITEMS("New Year's Day",
	"Epiphany",
	"Septuagesima Sunday",
	"Sexagesima Sunday",
	"Shrove Monday",
	"Shrove Tuesday",
        "Ash Wednesday",
	"Annunciation Day",
	"Passion Sunday",
	"Palm Sunday",
	"Holy Thursday",
	"Good Friday",
	"Easter",
	"Easter Sunday",
	"Easter Monday",
	"Rogation Sunday",
	"Ascencion Day",
	"Pentecost",
	"Whitsunday",
	"Whitmonday",
	"Trinity Sunday",
	"Corpus Christi",
	"Assumption Day",
	"Reformationstag",
	"All Saints' Day",
	"Repentance Day",
	"Totensonntag",
	"Advent Sunday",
	"nil",
	"Christmas",
	"Schweizer Bundesfeier",
	"Tag der Deutschen Einheit",
	 OE"sterreichischer Nationalfeiertag",
	"Augsburger Friedensfest",
        "Mother's Day",
	"Harvest Festival",
	"Valentine's Day",
	"St. Patrick's Day",
	"President's Day",
	"Armed Forces Day",
	"Memorial Day",
	"Father's Day",
	"Flag Day",
	"Independance Day",
	"Labor Day",
	"Columbus Day",
	"Halloween",
	"Election Day",
	"Veterans' Day",
	"Thanksgiving",
	),
  ITEMS("Neujahr",
	"Epiphanias",
	"Septuagesima",
	"Sexagesima",
	"Rosenmontag",
	"Fastnachtsdienstag",
        "Aschermittwoch",
	"Mari"ae" Verk"ue"ndigung",
	"Passionssonntag",
	"Palmsonntag",
	"Gr"ue"ndonnerstag",
	"Karfreitag",
	"Ostern",
	"Ostersonntag",
	"Ostermontag",
	"Rogate",
	"Himmelfahrt",
	"Pfingsten",
	"Pfingstsonntag",
	"Pfingstmontag",
	"Trinitatis",
	"Fronleichnam",
	"Mari"ae" Himmelfahrt",
	"Reformationstag",
	"Allerheiligen",
	"Bu"ss"tag",
	"Totensonntag",
	"Advent",
	"Mari"ae" Empf"ae"ngnis",
	"Weihnachten",
	"Schweizer Bundesfeier",
	"Tag der Deutschen Einheit",
	 OE"sterreichischer Nationalfeiertag",
	"Augsburger Friedensfest",
        "Muttertag",
	"Erntedankfest",
	"Valentinstag",
	"St. Patrick's Day",
	"President's Day",
	"Armed Forces Day",
	"Memorial Day",
	"Father's Day",
	"Flag Day",
	"Independance Day",
	"Labor Day",
	"Columbus Day",
	"Halloween",
	"Election Day",
	"Veterans' Day",
	"Thanksgiving",
       ),
);


static const FUNCLIST hebrew_holidays = SUBFUNCS
(
  FUNCS(EVT(B_HEBREW_JD0, 0), EVT(B_HEBREW_JD0, 0),
        EVT(B_HEBREW_JD0, 9),
	EVT(B_HEBREW_JD0, 14),
	EVT(B_HEBREW_JD0, 20),
	EVT(B_HEBREW_JD0, 21),
        EVT(B_HEBREW_JD0, 22),
	EVT(B_HEBREW_REVERSE, -163),
	EVT(B_HEBREW_REVERSE, -113),
	EVT(B_HEBREW_REVERSE, -193),
        EVT(B_HEBREW_KISLEV, 24),
	EVT(B_HEBREW_TEVET, 43),
	EVT2(B_HEBREW_JD0, 2, MOVE(SATURDAY, 1)),
	EVT(B_HEBREW_TEVET, 9),
	EVT2(B_HEBREW_REVERSE, -194, MOVE(SATURDAY, -2)),
	EVT2(B_HEBREW_REVERSE, -192, MOVE(SATURDAY, +1)),
	EVT(B_HEBREW_REVERSE, -72),
	EVT(B_HEBREW_REVERSE, -51),
	EVT(B_HEBREW_REVERSE, -45),
	EVT(B_HEBREW_REVERSE, -130),
	EVT(B_HEBREW_REVERSE, -143),
	EVT(B_HEBREW_REVERSE, -151),
       ),
  ITEMS("New Year's Day", "Rosh ha-Shanah",
        "Yom Kippur",
	"Sukkot",
	"Hoshana Rabba",
	"Shemini Azeret",
	"Simhat Torah",
	"Passover",
	"Shavout",
	"Purim",
	"Hanukkah",
	"Tu-B'Shevat",
	"Tzom Gedaliah",
	"Tzom Tevet",
	"Ta'anit Esther",
	"Shushan Purim",
	"Tzom Tammus",
	"Tisha-B'Av",
	"Tu-B'Av",
	"Lag B'Omer",
	"Yom ha-Azma'ut",
	"Yom ha-Shoah",
       ),
  ITEMS("Neujahr", "Rosh ha-Shanah",
        "Jom Kippur",
	"Sukkot",
	"Hoshana Rabba",
	"Shemini Azeret",
	"Simchat Torah",
	"Passah",
	"Shawuot",
	"Purim",
	"Chanukkah",
	"Tu bi-Schewat",
	"Zom Gedaljah",
	"Zom Tewet",
	"Ta'anit Esther",
	"Shushan Purim",
	"Zom Tammus",
	"Tisha bi-Aw",
	"Tu bi-Aw",
	"Lag bi-Omer",
	"Jom ha-Azma'ut",
	"Jom ha-Schoah",
       ),
);


static const FUNCLIST hindu_holidays = SUBFUNCS
(
  FUNCS(EVT(B_HINDU_JD0, 0)),
  ITEMS("New Year's Day"),
  ITEMS("Neujahr"),
);


static const FUNCLIST hindu2_holidays = SUBFUNCS
(
  FUNCS(EVT(B_HINDU2_JD0, 0)),
  ITEMS("New Year's Day"),
  ITEMS("Neujahr"),
);


static const FUNCLIST indian_holidays = SUBFUNCS
(
  FUNCS(EVT(B_INDIAN_JD0, 0)),
  ITEMS("New Year's Day"),
  ITEMS("Neujahr"),
);


static const FUNCLIST islamic_holidays = SUBFUNCS
(
  FUNCS(EVT(B_ISLAMIC_JD0, 0),
	EVT(B_ISLAMIC_JD0, 9),
	EVT(B_ISLAMIC_JD0, 70),
        EVT(B_ISLAMIC_JD0, 202),
	EVT(B_ISLAMIC_JD0, 221),
	EVT(B_ISLAMIC_JD0, 236),
	EVT(B_ISLAMIC_JD0, 262),
	EVT(B_ISLAMIC_JD0, 266),
	EVT(B_ISLAMIC_JD0, 334),
       ),
  ITEMS("New Year's Day",
	"Ashura",
	"Mawlid an-Nabi",
        "Lailat al-Mi'raj",
	"Lailat al-Bara't",
	"Ramadan",
        "Lailat al-Kadr",
	"Id al-Fitr",
	"Id al-Adha",
       ),
  ITEMS("Neujahr",
	"Aschura",
	"Meulud en-Nebi",
        "Lailet el-Miradsch",
	"Lailet el-Berat",
	"Ramadan",
        "Lailet el-Kadar",
	"Id el-Fitr",
	"Id el-Nahr",
       ),
);


static const FUNCLIST julian_holidays = SUBFUNCS
(
  FUNCS(EVT(B_JULIAN_JD0, 0), EVT(B_JULIAN_JD0, 5), EVT(B_JULIAN_EASTER, -63),
	EVT(B_JULIAN_EASTER, -56), EVT(B_JULIAN_EASTER, -48), EVT(B_JULIAN_EASTER, -47), 
        EVT(B_JULIAN_EASTER, -46), EVT(B_JULIAN_EASTER, -14), EVT(B_JULIAN_EASTER, -7),
	EVT(B_JULIAN_EASTER, -3), EVT(B_JULIAN_EASTER, -2), EVT(B_JULIAN_EASTER, 0),
	EVT(B_JULIAN_EASTER, +35), EVT(B_JULIAN_EASTER, +39), EVT(B_JULIAN_EASTER, +49),
	EVT(B_JULIAN_EASTER, +49), EVT(B_JULIAN_EASTER, +50), EVT(B_JULIAN_EASTER, +56),
        EVT(B_JULIAN_EASTER, +60), EVT(B_JULIAN_REVERSE, -139),
	EVT2(B_JULIAN_REVERSE, -29, 7), EVT(B_JULIAN_REVERSE, -7),
       ),
  ITEMS("New Year's Day", "Epiphany", "Septuagesima Sunday",
	"Sexagesima Sunday", "Shrove Monday", "Shrove Tuesday",
        "Ash Wednesday", "Passion Sunday", "Palm Sunday",
	"Holy Thursday", "Good Friday", "Easter",
	"Rogation Sunday", "Ascencion Day", "Pentecost",
	"Whitsunday", "Whitmonday", "Trinity Sunday",
	"Corpus Christi", "Assumption Day",
	"Advent Sunday", "Christmas",
       ),
  ITEMS("Neujahr", "Epiphanias", "Septuagesima",
	"Sexagesima", "Rosenmontag", "Fastnachtsdienstag",
        "Aschermittwoch", "Passionssonntag", "Palmsonntag",
	"Gr"ue"ndonnerstag", "Karfreitag", "Ostern",
	"Rogate", "Himmelfahrt", "Pfingsten",
	"Pfingstsonntag", "Pfingstmontag", "Trinitatis", 
	"Fronleichnam", "Mari"ae" Himmelfahrt",
	"Advent", "Weihnachten",
       ),
);


static const FUNCLIST mayan_holidays = SUBFUNCS
(
  FUNCS(EVT(B_MAYAN_JD0, 0),
	EVT(B_MAYAN_JD0, 0),
	EVT(B_MAYAN_HAAB, 0),
	EVT(B_MAYAN_TZOLKIN, 0)
	),
  ITEMS("New Year's Day",
	"New cycle",
	"Haab",
	"Tzolkin"),
  ITEMS("Neujahr",
	"Zyklenwechsel",
	"Haab",
	"Tzolkin"
	),
);


static const FUNCLIST persian_holidays = SUBFUNCS
(
  FUNCS(EVT(B_PERSIAN_JD0, 0)),
  ITEMS("New Year's Day"),
  ITEMS("Neujahr"),
);


static const FUNCLIST seleucid_holidays = SUBFUNCS
(
  FUNCS(EVT(B_SELEUCID_JD0, 0)),
  ITEMS("New Year's Day"),
  ITEMS("Neujahr"),
);


static const FUNCLIST syrian_holidays = SUBFUNCS
(
  FUNCS(EVT(B_SYRIAN_JD0, 0)),
  ITEMS("New Year's Day"),
  ITEMS("Neujahr"),
);


static const FUNCLIST taiwanese_holidays = SUBFUNCS
(
  FUNCS(EVT(B_TAIWANESE_JD0, 0)),
  ITEMS("New Year's Day"),
  ITEMS("Neujahr"),
);


static const FUNCLIST yazdegerd_holidays = SUBFUNCS
(
  FUNCS(EVT(B_YAZDEGERD_JD0, 0)),
  ITEMS("New Year's Day"),
  ITEMS("Neujahr"),
);


static const FUNCLIST holidays = SUBFUNCS
(
  FUNCS((POINTER)&armenian_holidays, (POINTER)&astronomy,
	(POINTER)&buddhist_holidays, (POINTER)&byzantine_holidays,
        (POINTER)&chinese_holidays, (POINTER)&coptic_holidays,
  	(POINTER)&egyptian_holidays, (POINTER)&ethiopic_holidays,
	(POINTER)&french_holidays, (POINTER)&gengou_holidays,
	(POINTER)&gregorian_holidays, (POINTER)&hebrew_holidays,
	(POINTER)&hindu_holidays, (POINTER)&hindu2_holidays,
  	(POINTER)&indian_holidays, (POINTER)&islamic_holidays,
	(POINTER)&iso8601_holidays, (POINTER)&julian_holidays,
	(POINTER)&roman_holidays, (POINTER)&mayan_holidays,
	(POINTER)&persian_holidays, (POINTER)&seleucid_holidays,
	(POINTER)&syrian_holidays,
	(POINTER)&taiwanese_holidays, (POINTER)&yazdegerd_holidays,
	),
  calendar_names_en,
  calendar_names_de,
);


static const FUNCLIST armenian_monnames =
{
  13,
  NULL,
  armenian_month,
  armenian_month,
};


static const FUNCLIST chinese_monnames =
{
  12,
  NULL,
  monname_en,
  monname_de,
};


static const FUNCLIST coptic_monnames =
{
  13,
  NULL,
  coptic_month,
  coptic_month,
};


static const FUNCLIST ethiopic_monnames =
{
  13,
  NULL,
  ethiopic_month,
  ethiopic_month,
};


static const FUNCLIST egyptian_monnames =
{
  13,
  NULL,
  egyptian_month,
  egyptian_month,
};


static const FUNCLIST french_monnames =
{
  13,
  NULL,
  french_month,
  french_month,
};


static const FUNCLIST gengou_names =
{
  16,
  NULL,
  gengou_era,
  gengou_era,
};


static const FUNCLIST hebrew_monnames =
{
  13,
  NULL,
  hebrew_month_en,
  hebrew_month_de,
};


static const FUNCLIST hindu_monnames =
{
  12,
  NULL,
  hindu_month,
  hindu_month,
};


static const FUNCLIST hindu2_monnames =
{
  12,
  NULL,
  hindu2_month,
  hindu2_month,
};


static const FUNCLIST indian_monnames =
{
  12,
  NULL,
  indian_month,
  indian_month,
};


static const FUNCLIST islamic_monnames =
{
  12,
  NULL,
  islamic_month_en,
  islamic_month_de,
};


static const FUNCLIST julian_monnames =
{
  12,
  NULL,
  monname_en,
  monname_de,
};


static const FUNCLIST persian_monnames =
{
  12,
  NULL,
  persian_month,
  persian_month,
};


static const FUNCLIST roman_daynames =
{
  20,
  NULL,
  roman_day,
  roman_day,
};

static const FUNCLIST roman_fixpoints = SUBFUNCS
(
  NULL,
  ITEMS("Nonas", "Idus", "Kalendas"), 
  ITEMS("Nonas", "Idus", "Kalendas"), 
);


static const FUNCLIST roman_monnames =
{
  12,
  NULL,
  roman_month,
  roman_month,
};


static const FUNCLIST syrian_monnames =
{
  12,
  NULL,
  syrian_month,
  syrian_month,
};


static const FUNCLIST yazdegerd_monnames =
{
  13,
  NULL,
  yazdegerd_month,
  yazdegerd_month,
};


static const FUNCLIST void_monnames = SUBFUNCS
(
  NULL,
  ITEMS("1st", "2nd", "3rd", "4th", "5th", "6th", "7th", "8th", "9th", "10th", "11th", "12th", "13th"), 
  ITEMS("1.", "2.", "3.", "4.", "5.", "6.", "7.", "8.", "9.", "10.", "11.", "12.", "13."), 
);


static EVENT baseevents[] =
{
  NULL,			/* Reihenfolge wie in BASEEVENTS! */
  gregorian_jd0,
  gregorian_easter,
  gregorian_reverse,
  julian_jd0,
  julian_easter,
  julian_reverse,

  armenian_jd0,
  buddhist_jd0,
  byzantine_jd0,
  chinese_jd0,
  chinese_dongzhi,
  chinese_xiazhi,
  chinese_chushu,
  coptic_jd0,
  coptic_easter,
  egyptian_jd0,
  ethiopic_jd0,
  ethiopic_easter,
  french_jd0,
  gengou_jd0,
  hebrew_jd0,
  hebrew_kislev,
  hebrew_tevet,
  hebrew_reverse,
  hindu_jd0,
  hindu2_jd0,
  indian_jd0,
  islamic_jd0,
  iso_jd0,
  mayan_jd0,
  mayan_haab,
  mayan_tzolkin,
  persian_jd0,
  seleucid_jd0,
  syrian_jd0,
  taiwanese_jd0,
  yazdegerd_jd0,
};


/*
 * Julianische Tageszahl JD
 */
static int astronomical_sprintf(char *buf, double jd, int mode)
{
  return sprintf(buf, "%f", jd);
}


/*
 * Standard-Kalender
 */
static int calendar_sprintf(char *buf, double jd, int mode)
{
  if (jd < jdcalref+0.5)
    return julian_sprintf(buf, jd, mode);

  return gregorian_sprintf(buf, jd, mode);
}


/*
 * Berechnet wird JD vom koptischen Osterfest
 */
static double coptic_easter(long year)
{
  return julian_easter(year + 284) - 0.25;
}


/*
 * Berechnet wird JD vom thiopischen Osterfest
 */
static double ethiopic_easter(long year)
{
  return julian_easter(year + 8) - 0.25;
}


/*
 * Berechnet wird JD von Neujahr des folgenden! Jahres
 */
static double gregorian_reverse(long year)
{
  return gregorian_jd0(year + 1);	/* Neujahr des folgenden Jahres */
}


/*
 * Berechnet wird JD von Neujahr des folgenden! juedischen Jahres
 */
static double hebrew_reverse(long year)
{
  return hebrew_jd0(year + 1);		/* Neujahr des folgenden Jahres */
}


/*
 * JD von Neujahr des folgenden! julianischen Jahres
 */
static double julian_reverse(long year)
{
  return julian_jd0(year + 1);		/* Neujahr des folgenden Jahres */
}


/*
 * Berechnet das Auftreten des naechsten Ereignisses ab JD = 'jd',
 * dass durch 'data' und 'conv' definiert wird.
 */
static double next_event(double jd, long data, void *conv)
{
  double j, j0;

  j = j0 = jd;

  if (!BASE(data)) {				/* kein Basisereignis ? */
    j = astronomical(j, OFFSET(data), PHASE(data));
  }
  else {
    for (;; j0 += j0 - j) {			/* keine Ereignisse vor Startdatum */

      j = jd_event(j0 - OFFSET(data), conv, baseevents[BASE(data)]);	/* JD des Basisereignisses */

      if (PHASE(data)) {				/* Phasenverschiebung ? */
        if (SHIFT(data)) {
          if (AVOID(data) == imod(itrunc(j + _half + OFFSET(data)), 7))
            j += SHIFT(data);
        }
        else
          j -= fmod(PHASE(data) + j + _half, 7);
      }

      j += OFFSET(data);

      if (j >= j0)
        break;			/* erreicht */
    }
  }

  return j;
}


/*
 * Addin-Funktionen auf Landessprache einstellen
 */
void init_addin(int lang)
{
  switch (lang) {

    case LANGUAGE_ENGLISH:
      calref = 17520903;		/* englische Kalenderreform */
      jdcalref = 2361222;
      dayname = dayname_en;
      monname = monname_en;
      chinese_zodiac = chinese_zodiac_en;
      hebrew_day = hebrew_day_en;
      hebrew_month = hebrew_month_en;
      islamic_day = islamic_day_en;
      islamic_month = islamic_month_en;
      break;
    
    case LANGUAGE_GERMAN:
      dayname = dayname_de;
      monname = monname_de;
      chinese_zodiac = chinese_zodiac_de;
      hebrew_day = hebrew_day_de;
      hebrew_month = hebrew_month_de;
      islamic_day = islamic_day_de;
      islamic_month = islamic_month_de;
      break;

    default:
      break;
  }
}


SO_QUIRKSCALL(SC_armenian, double *r, double *year, char *month, double *day)
{
  DATE d;

  d.day = *day;
  d.month = (int)scan(month, &armenian_monnames);

  if (d.day < 1 || d.day >= 31 || d.month < 1 || d.month > 13) {	/* Fehler ? */
    *r = INVALID;
    return;
  }

  d.year = itrunc(*year);

  *r = JD2SC(armenian_jd(&d));
}


SO_CALLBACK(SC_astronomy, double *rc, double *date)
{
  *rc = JD2SC(*date);
}


SO_QUIRKSCALL(SC_buddhist, double *r, double *year, char *month, double *day)
{
  DATE d;

  d.day = *day;
  d.month = (int)scan(month, &gregorian_monnames);

  if (d.day < 1 || d.day >= 32 || d.month < 1 || d.month > 12) {	/* Fehler ? */
    *r = INVALID;
    return;
  }

  d.year = itrunc(*year);

  *r = JD2SC(buddhist_jd(&d));
}


SO_QUIRKSCALL(SC_byzantine, double *r, double *year, char *month, double *day)
{
  DATE d;

  d.day = *day;
  d.month = (int)scan(month, &julian_monnames);

  if (d.day < 1 || d.day >= 32 || d.month < 1 || d.month > 12) {	/* Fehler ? */
    *r = INVALID;
    return;
  }

  d.year = itrunc(*year);

  *r = JD2SC(byzantine_jd(&d));
}


SO_QUIRKSCALL(SC_calendar, char *r, char *name, double *date, double *mode)
{
  double d = SC2JD(*date);
  SPRINTF func;

  if (d < JD_MIN || d > JD_MAX) {	/* Fehler ? */
    *r = INVALID;
    return;
  }

  func = scan(name, &calendar); 

  if (!func)
    func = calendar_sprintf;		/* Standard-Kalender */

  func(r, d, itrunc(*mode));		/* Kalenderdatum ausgeben */
}


SO_CALLBACK(SC_chronology, char *r, char *name, double *year)
{
  long y = itrunc(*year);
  int c = (int)scan(name, &chronology) - 1;

  switch (c) {

    case C_EPACT:
      sprintf(r, "%d", epact(y));
      return;

    case C_INDICTION:
      sprintf(r, "%d", indiction(y));
      return;

    case C_REGENT:
      SO_StringCopy(r, regent(y));
      return;

    case C_AUREUS:
      SO_StringCopy(r, aureus(y));
      return;

    case C_SUNCIRC:
      sprintf(r, "%d", suncirc(y));
      return;

    case C_DOMINICAL:
      SO_StringCopy(r, dominical(y));
      return;
  }

  SO_StringCopy(r, INVALID_ARG);
}


SO_QUIRKSCALL(SC_chinese, double *r, double *cycle, double *year, char *month, double *leap, double *day)
{
  LEAPDATE d;
  long c = itrunc(*cycle), n = itrunc(*year);

  d.day = *day;
  d.month = (int)scan(month, &void_monnames);

  if (n < 1 || n > 60 || d.day < 1 || d.day >= 31 || d.month < 1 || d.month > 12) {	/* Fehler ? */
    *r = INVALID;
    return;
  }

  d.leap = (*leap != 0);

#if 0
  if (!n)				/* lineare Zaehlung ? */
    d.year = c;
  else					/* sonst zyklkische Zaehlung */
#endif
    d.year = CHINESE_YEAR(c, n);

  *r = JD2SC(chinese_jd((pDATE)&d));
}


SO_QUIRKSCALL(SC_coptic, double *r, double *year, char *month, double *day)
{
  DATE d;

  d.day = *day;
  d.month = (int)scan(month, &coptic_monnames);

  if (d.day < 1 || d.day >= 31 || d.month < 1 || d.month > 13) {	/* Fehler ? */
    *r = INVALID;
    return;
  }

  d.year = itrunc(*year);

  *r = JD2SC(coptic_jd(&d));
}


SO_QUIRKSCALL(SC_egyptian, double *r, double *year, char *month, double *day)
{
  DATE d;

  d.day = *day;
  d.month = (int)scan(month, &egyptian_monnames);

  if (d.day < 1 || d.day >= 31 || d.month < 1 || d.month > 13) {	/* Fehler ? */
    *r = INVALID;
    return;
  }

  d.year = itrunc(*year);

  *r = JD2SC(egyptian_jd(&d));
}


SO_QUIRKSCALL(SC_ethiopic, double *r, double *year, char *month, double *day)
{
  DATE d;

  d.day = *day;
  d.month = (int)scan(month, &ethiopic_monnames);

  if (d.day < 1 || d.day >= 31 || d.month < 1 || d.month > 13) {	/* Fehler ? */
    *r = INVALID;
    return;
  }

  d.year = itrunc(*year);

  *r = JD2SC(ethiopic_jd(&d));
}


SO_QUIRKSCALL(SC_event, double *r, double *ord, char *dname, char *month, double *year)
{
  DATE date;
  double j;
  int m, n, o;
  
  o = itrunc(*ord);
  n = (int)scan(dname, &day_names);	/* Wochentag {1=Mo, ..., 7=So} */
  m = (int)scan(month, &gregorian_monnames);

  if (m == 0)			/* Von Jahresbeginn/ende an zaehlen ? */
    m = (o > 0)? 1: 12;

  if (o < -366 || o > 366 || n < 0 || n > 7 || m < 1 || m > 12) {
    *r = INVALID;
    return;
  }

  date.year  = itrunc(*year);
  date.month = m;

  if (n) {			/* Tagesname ? */
    o *= 7;
  
    if (o <= 0)					/* vom Monatsende aus rechnen? */
      o += monthlen(date.year)[m-1];

    date.day = o;
    
    j = jd(&date) + _half;			/* normalisieren */

    *r = JD2SC(j + fmod(n - 8 - fmod(j, 7), 7)) - _half;
  }
  else {

    if (o <= 0)					/* vom Monatsende aus rechnen? */
      o += monthlen(date.year)[m-1];

    date.day = o;
    
    *r = JD2SC(jd(&date));
  }
}


SO_QUIRKSCALL(SC_french, double *r, double *year, char *month, double *day)
{
  DATE d;

  d.day = *day;
  d.month = (int)scan(month, &french_monnames);

  if (d.day < 1 || d.day >= 31 || d.month < 1 || d.month > 13) {	/* Fehler ? */
    *r = INVALID;
    return;
  }

  d.year = itrunc(*year);

  *r = JD2SC(french_jd(&d));
}


SO_QUIRKSCALL(SC_gengou, double *r, char *era, double *year, char *month, double *day)
{
  GENGOUDATE g;

  g.day = *day;
  g.month = (int)scan(month, &gregorian_monnames);
  g.era = (int)scan(era, &gengou_names);

  if (g.era)
    g.era--;		/* 0-basiert */

  if (g.day < 1 || g.day >= 32 || g.month < 1 || g.month > 12) {	/* Fehler ? */
    *r = INVALID;
    return;
  }

  g.year = itrunc(*year);

  *r = JD2SC(gengou_jd((pDATE)&g));
}


SO_CALLBACK(SC_gnomon, double *r, double *date)
{
  double d = frac(*date);				/* Tagesbruchzahl */

  *r = d + equotime(SC2TD(*date)) / F_TWOPI;		/* Zeitgleichung */
}


SO_QUIRKSCALL(SC_gregorian, double *r, double *year, char *month, double *day)
{
  DATE d;

  d.day = *day;
  d.month = (int)scan(month, &gregorian_monnames);

  if (d.day < 1 || d.day >= 32 || d.month < 1 || d.month > 12) {	/* Fehler ? */
    *r = INVALID;
    return;
  }

  d.year = itrunc(*year);

  *r = JD2SC(gregorian_jd(&d));
}


SO_QUIRKSCALL(SC_hebrew, double *r, double *year, char *month, double *day)
{
  DATE d;
 
  d.day = *day;
  d.month = (int)scan(month, &hebrew_monnames);

  if (d.day < 1 || d.day >= 31 || d.month < 1 || d.month > 13) {	/* Fehler ? */
    *r = INVALID;
    return;
  }

  d.year = itrunc(*year);

  *r = JD2SC(hebrew_jd(&d));
}


SO_QUIRKSCALL(SC_hindu, double *r, double *year, char *month, double *leap, double *day)
{
  LEAPDATE d;
 
  d.day = *day;
  d.month = (int)scan(month, &hindu_monnames);

  if (d.day < 1 || d.day >= 31 || d.month < 1 || d.month > 12) {	/* Fehler ? */
    *r = INVALID;
    return;
  }

  d.leap = (*leap != 0);
  d.year = itrunc(*year);

  *r = JD2SC(hindu_jd((pDATE)&d));
}


SO_QUIRKSCALL(SC_hindu2, double *r, double *year, char *month, double *day)
{
  DATE d;
 
  d.day = *day;
  d.month = (int)scan(month, &hindu2_monnames);

  if (d.day < 1 || d.day >= 32 || d.month < 1 || d.month > 12) {	/* Fehler ? */
    *r = INVALID;
    return;
  }

  d.year = itrunc(*year);

  *r = JD2SC(hindu2_jd(&d));
}


SO_QUIRKSCALL(SC_holiday, double *r, char *cname, char *ename, char *epoch)
{
  double e, j;
  char *p0, *p1;
  long data, e0, e1;
  FUNCLIST *elist = scan(cname, &holidays);	/* Festtagsliste laden */

  if (!elist)					/* kein Kalender? dann Standard */
    elist = (FUNCLIST*)&gregorian_holidays;

  *r = INVALID;					/* vorbelegen */

  e0 = strtol(epoch, &p0, 10);			/* Epoche holen */

  if (epoch == p0)				/* Fehler ? */
    return;

  if (elist == &chinese_holidays) {		/* zusammengesetzte Epoche ? */
    e1 = strtol(p0+1, &p1, 10);
    if (p0+1 == p1)				/* Fehler */
      return;
    
    e = CHINESE_YEAR(e0, e1);
  }
  else
    e = e0;

  data = (long)scan(ename, elist);

  if (!data)					/* Ereignis unbekannt ? */
    return;

  if (!BASE(data)) {				/* kein Basisereignis ? */
    *r = JD2SC(astronomical(jd0(e), OFFSET(data) | 0x10000, PHASE(data)));
    return;
  }

  j = baseevents[BASE(data)](itrunc(e));	/* JD des Basisereignisses */

  if (PHASE(data)) {				/* Phasenverschiebung ? */
    if (SHIFT(data)) {
      if (AVOID(data) == imod(itrunc(j + _half + OFFSET(data)), 7))
        j += SHIFT(data);
    }
    else
      j -= fmod(PHASE(data) + j + _half, 7);
  }

  *r = JD2SC(j) + OFFSET(data);			/* JD des Ereignisses */
}


SO_QUIRKSCALL(SC_holiday2, double *r, char *cname, char *ename, double *start)
{
  double s = SC2JD(*start);
  long data;
  void *conv;
  FUNCLIST *elist = scan(cname, &holidays);	/* Festtagsliste laden */

  if (!elist) {					/* kein Kalender? dann Standard */
    elist = (FUNCLIST*)&gregorian_holidays;
    conv = gregorian_caldate;
  }
  else
   conv = scan(cname, &jd2other);		/* Umrechnung: jd -> otherdate */

  data = (long)scan(ename, elist);

  if (!data || s < JD_MIN || s > JD_MAX) {		/* Fehler ? */
    *r = INVALID;
    return;
  }

  *r = JD2SC(next_event(s, data, conv));
}


SO_QUIRKSCALL(SC_indian, double *r, double *year, char *month, double *day)
{
  DATE d;
 
  d.day = *day;
  d.month = (int)scan(month, &indian_monnames);

  if (d.day < 1 || d.day >= 31 || d.month < 1 || d.month > 12) {	/* Fehler ? */
    *r = INVALID;
    return;
  }

  d.year = itrunc(*year);

  *r = JD2SC(indian_jd(&d));
}


SO_QUIRKSCALL(SC_islamic, double *r, double *year, char *month, double *day)
{
  DATE d;
  
  d.day = *day;
  d.month = (int)scan(month, &islamic_monnames);

  if (d.day < 1 || d.day >= 31 || d.month < 1 || d.month > 12) {	/* Fehler ? */
    *r = INVALID;
    return;
  }

  d.year = itrunc(*year);

  *r = JD2SC(islamic_jd(&d));
}


SO_QUIRKSCALL(SC_iso8601, double *r, double *year, double *week, char *day)
{
  DATE d;
  
  if ((d.day = strtod(day, NULL)) == 0)
    d.day = (double)(int)scan(day, &day_names);

  d.month = itrunc(*week);

  if (d.day < 1 || d.day >= 8 || d.month < 1 || d.month > 53) {	/* Fehler ? */
    *r = INVALID;
    return;
  }

  d.year = itrunc(*year);

  *r = JD2SC(iso_jd(&d));
}


SO_QUIRKSCALL(SC_julian, double *r, double *year, char *month, double *day)
{
  DATE d;

  d.day = *day;
  d.month = (int)scan(month, &julian_monnames);

  if (d.day < 1 || d.day >= 32 || d.month < 1 || d.month > 12) {	/* Fehler ? */
    *r = INVALID;
    return;
  }

  d.year = itrunc(*year);

  *r = JD2SC(julian_jd(&d));
}


SO_QUIRKSCALL(SC_latin, double *j, double *year, char *month, char *fix, char *day)
{
  ROMANDATE r;
  double d;
  int f, m;

  if ((d = strtod(day, NULL)) == 0) {
    f = (int)scan(day, &roman_daynames);
    d = (f > 19)? 0: f;
  }
  f = (int)scan(fix, &roman_fixpoints);
  m = (int)scan(month, &roman_monnames);

  if (f < 1 || f > 3 || m < 1 || m > 12 || d < 0 || d >= 21) {	/* Fehler ? */
    *j = INVALID;
    return;
  }
  
  r.year = itrunc(*year);
  r.month = m;
  r.fix = f - 1;
  r.day = d;					/* TODO: Tagesbruchzahl */
  
  *j = JD2SC(roman_jd((pDATE)&r));
}


SO_CALLBACK(SC_leapyear, double *r, char *name, char *epoch)
{
  char *p0, *p1;
  long e0, e1;
  LEAPYEAR leap = scan(name, &leapyear); 

  if (!leap)
    leap = isleap;				/* Standard */

  *r = INVALID;					/* vorbelegen */

  e0 = strtol(epoch, &p0, 10);			/* Epoche holen */

  if (epoch == p0)				/* Fehler ? */
    return;

  if (leap == chinese_isleap) {			/* zusammengesetzte Epoche ? */
    e1 = strtol(p0+1, &p1, 10);
    if (p0+1 == p1)				/* Fehler */
      return;
    
    e0 = CHINESE_YEAR(e0, e1);
  }

  *r = leap(e0);				/* auf Schaltjahr testen */
}


SO_CALLBACK(SC_leapyear2, double *r, char *name, double *date)
{
  DATE d;
  CALDATE conv = scan(name, &jd2other);
  LEAPYEAR leap = scan(name, &leapyear); 

  if (!conv || !leap) {
    conv = caldate;				/* Standard-Kalender */
    leap = isleap;
  }

  conv(SC2JD(*date), &d);			/* Jahreszahl bestimmen */

  *r = leap(d.year);				/* auf Schaltjahr testen */
}


SO_QUIRKSCALL(SC_mayan, double *r, double *baktun, double *katun, double *tun,
                        double *uinal, double *kin)
{
  MAYADATE m;

  m.alautun = m.kinchiltun = m.calabtun = m.pictun = 0;
  m.baktun = itrunc(*baktun);
  m.katun = itrunc(*katun);
  m.tun = itrunc(*tun);
  m.uinal = itrunc(*uinal);
  m.kin = itrunc(*kin);

  *r = JD2SC(mayan_jd((pDATE)&m));
}


SO_QUIRKSCALL(SC_persian, double *r, double *year, char *month, double *day)
{
  DATE d;

  d.day = *day;
  d.month = (int)scan(month, &persian_monnames);

  if (d.day < 1 || d.day >= 32 || d.month < 1 || d.month > 12) {	/* Fehler ? */
    *r = INVALID;
    return;
  }

  d.year = itrunc(*year);

  *r = JD2SC(persian_jd(&d));
}


SO_QUIRKSCALL(SC_seleucid, double *r, double *year, char *month, double *day)
{
  DATE d;

  d.day = *day;
  d.month = (int)scan(month, &julian_monnames);

  if (d.day < 1 || d.day >= 32 || d.month < 1 || d.month > 12) {	/* Fehler ? */
    *r = INVALID;
    return;
  }

  d.year = itrunc(*year);

  *r = JD2SC(seleucid_jd(&d));
}


SO_QUIRKSCALL(SC_syrian, double *r, double *year, char *month, double *day)
{
  DATE d;

  d.day = *day;
  d.month = (int)scan(month, &syrian_monnames);

  if (d.day < 1 || d.day >= 32 || d.month < 1 || d.month > 12) {	/* Fehler ? */
    *r = INVALID;
    return;
  }

  d.year = itrunc(*year);

  *r = JD2SC(syrian_jd(&d));
}


SO_QUIRKSCALL(SC_taiwanese, double *r, double *year, char *month, double *day)
{
  DATE d;

  d.day = *day;
  d.month = (int)scan(month, &gregorian_monnames);

  if (d.day < 1 || d.day >= 32 || d.month < 1 || d.month > 12) {	/* Fehler ? */
    *r = INVALID;
    return;
  }

  d.year = itrunc(*year);

  *r = JD2SC(taiwanese_jd(&d));
}


SO_QUIRKSCALL(SC_when, double *r, char *cname, char *ename,
              char *dname, double *day, char *month, double *start)
{
  DATE date;
  double b, e, j, jt;
  long c, data;					/* c = Counter */
  void *conv;
  int n, moment, d = itrunc(*day);
  int m = (int)scan(month, &gregorian_monnames);
  FUNCLIST *elist = scan(cname, &holidays);	/* Festtagsliste laden */

  *r = INVALID;					/* vorbelegen */

  if (d < 0 || d >= 32 || m < 0 || m > 13)	/* Fehler ? (TODO: verbessern) */
    return;

  if (!elist) {					/* kein Kalender? dann Standard */
    elist = (FUNCLIST*)&gregorian_holidays;
    conv = gregorian_caldate;
  }
  else
   conv = scan(cname, &jd2other);		/* Umrechnung: jd -> otherdate */

  n     = (int)scan(dname, &day_names);		/* Wochentag {1=Mo, ..., 7=So} */
  data  = (long)scan(ename, elist);		/* Ereignisdefinition laden */

  if (!BASE(data) && elist != &astronomy && elist != &gregorian_holidays)	/* kein Basisereignis ? */
    return;

  moment = (elist == &astronomy);

  e = data? SC2JD(*start): SC2JD(floor(*start));	/* wenn Basisereignis fehlt, am Tagesanfang beginnen */

  if (e < JD_MIN)					/* vor zulaessigem Bereich ? */
    return;

  for (c = 0; c < _max_loop && e < JD_MAX; c++, e = b + 1) {

    if (data) {
      if (!BASE(data)) {			/* astronomisch? */

        j = astronomical(e, OFFSET(data), PHASE(data));

        b = j + 25;				/* Beschleunigung */
      }
      else {
        b = j = jd_event(e, conv, baseevents[BASE(data)]);	/* JD des Basisereignisses */

        if (PHASE(data))				/* Phasenverschiebung ? */
          j -= fmod(j + PHASE(data) + _half, 7);

        j += OFFSET(data);	    		/* JD des Ereignisses */
      }

      if (j < e) {				/* vor dem Startdatum? */
        b += e - j;		/* "festfahren" vermeiden */
        continue;
      }
    }
    else		/* Basisereignis fehlt, also Start-JD verwenden */
      b = j = e;

    if (moment) {		/* punktuelles Ereignis ? */

      if (n)				/* Tagesname suchen... ? */
        if (n != imod(itrunc(j + 0.5), 7) + 1)
          continue;				/* weitersuchen */ 

      caldate(j, &date);		/* Tag und Monat suchen: Datum bestimmen */

      if (d == 0)		/* d == 0 ? dann den Tag nicht vergleichen */
        date.day = 0;

      if (m == 0)		/* m == 0 ? dann den Monat nicht vergleichen */
        date.month = 0;

      if (m != date.month || d != itrunc(date.day))
        continue;					/* weitersuchen */
    }
    else {	/* ueberlappender Bereich, verglichen werden die Tagesanfaenge */

      caldate(j, &date);		/* Tag und Monat suchen: Datum bestimmen */

      if (d)			/* Tag vergleichen ? */
        date.day = d;

      if (m)			/* Monat vergleichen ? */
        date.month = m;

      jt = j - jd(&date);	/* Differenz der Daten */

      if (fabs(jt) >= 1.0)	/* nicht am gleichen Datum (keine Ueberschneidung) ? */
        continue;

      if (n) {			/* Tagesnamen suchen ? */
        if (jt < 0) {					/* liegt Basisereignis vor Tagesbeginn ? */
          if (n != imod(ifloor(j + 1.5), 7) + 1)	/* dann naechsten Tag testen */
	    continue;
	}
	else {
          if (n != imod(ifloor(j + 0.5), 7) + 1)	/* stimmt Tagesname ? */
            continue;  
	}
      }
//      j = floor(j) + 0.5;		/* den Tagesanfang zurueckliefern */
    }

    *r = JD2SC(j);    /* gefunden! */
    return;
  }
}


/*
 * Die Funktion berechnet, wann zwei Ereignisse zweier Kalender
 * aufeinander fallen. 'Aufeinander fallen' ist hierbei definiert als:
 *
 *	Die Ereignisse liegen weniger als 1.0 Tage auseinander.
 *
 *	Astronom. Ereignisse duerfen nicht vor einem Kalenderereignis liegen. 
 *
 * Hierdurch werden auch Kalender, die nicht um Mitternacht das Datum
 * wechseln, sowie astronomische Ereignisse erfasst.
 *
 * Zurckgeliefert wird der Zeitpunkt des 1. Ereignisses.
 */
SO_QUIRKSCALL(SC_when2, double *r, char *cname1, char *ename1,
              char *cname2, char *ename2, double *start)
{
  double j1, j2;
  long c, data1, data2;
  void *conv1, *conv2;
  FUNCLIST *list1, *list2;

  list1 = scan(cname1, &holidays);		/* Festtagsliste laden */

  if (!list1) {					/* kein Kalender? dann Standard */
    list1 = (FUNCLIST*)&gregorian_holidays;
    conv1 = gregorian_caldate;
  }
  else
   conv1 = scan(cname1, &jd2other);		/* Umrechnung: jd -> otherdate */

  list2 = scan(cname2, &holidays);		/* Festtagsliste laden */

  if (!list2) {					/* kein Kalender? dann Standard */
    list2 = (FUNCLIST*)&gregorian_holidays;
    conv2 = gregorian_caldate;
  }
  else
   conv2 = scan(cname2, &jd2other);		/* Umrechnung: jd -> otherdate */

  *r = INVALID;					/* vorbelegen */

  data1  = (long)scan(ename1, list1);		/* Ereignisdefinitionen laden */
  data2  = (long)scan(ename2, list2);

  j1 = j2 = SC2JD(*start);				/* Start-JD */ 

  if (!data1 || !data2 || j1 < JD_MIN)			/* Fehler ? */
    return; 

  j1 = next_event(j1, data1, conv1);
  j2 = next_event(j2, data2, conv2);			/* Startwerte */

  for (c = 0; c < _max_loop && j1 < JD_MAX; c++) {

    if (fabs(j1 - j2) < 1.0 && (list1 == list2 ||
        (j1 <= j2 && list1 != &astronomy) || (j2 <= j1 && list2 != &astronomy))) {
      *r = JD2SC(j1);
      return;
    }

    if (j1 < j2)
      j1 = next_event(j1 + 1, data1, conv1);
    else
      j2 = next_event(j2 + 1, data2, conv2);
  }
}


SO_QUIRKSCALL(SC_yazdegerd, double *r, double *year, char *month, double *day)
{
  DATE d;

  d.day = *day;
  d.month = (int)scan(month, &yazdegerd_monnames);

  if (d.day < 1 || d.day >= 31 || d.month < 1 || d.month > 13) {	/* Fehler ? */
    *r = INVALID;
    return;
  }

  d.year = itrunc(*year);

  *r = JD2SC(yazdegerd_jd(&d));
}


